@@ -26,7 +26,7 @@
 #include <WINGs/WUtil.h>
 #include <X11/Xlib.h>
 
-#define WINGS_H_VERSION  20041030
+#define WINGS_H_VERSION  20150508
 
 
 #ifdef __cplusplus
@@ -37,6 +37,22 @@ extern "C" {
 #endif
 
 
+#ifdef __STDC_VERSION__
+# if __STDC_VERSION__ >= 201112L
+/*
+ * Ideally, we would like to include the proper header to have 'noreturn' properly
+ * defined (that's what is done for the rest of the code)
+ * However, as we're a public API file we can't do that in a portable fashion, so
+ * we just stick to plain STD C11 keyword
+ */
+#  define _wings_noreturn _Noreturn
+# else
+#  define _wings_noreturn /**/
+# endif
+#else
+#define _wings_noreturn /**/
+#endif
+
 typedef unsigned long WMPixel;
 
 
@@ -111,7 +127,8 @@ typedef enum {
     WBTRadio = 5,
     WBTMomentaryChange = 6,
     WBTOnOff = 7,
-    WBTMomentaryLight = 8
+    WBTMomentaryLight = 8,
+    WBTTriState = 9
 } WMButtonType;
 
 /* button behaviour masks */
@@ -626,9 +643,11 @@ WMRect wmkrect(int x, int y, unsigned int width, unsigned int height);
 /* ---[ WINGs/wapplication.c ]-------------------------------------------- */
 
 
-
 void WMInitializeApplication(const char *applicationName, int *argc, char **argv);
 
+/* You're supposed to call this funtion before exiting so WINGs can terminate properly */
+void WMReleaseApplication(void);
+
 void WMSetResourcePath(const char *path);
 
 /* don't free the returned string */
@@ -648,7 +667,7 @@ WMScreen* WMCreateScreen(Display *display, int screen);
 
 WMScreen* WMCreateSimpleApplicationScreen(Display *display);
 
-void WMScreenMainLoop(WMScreen *scr);
+_wings_noreturn void WMScreenMainLoop(WMScreen *scr);
 
 void WMBreakModalLoop(WMScreen *scr);
 
@@ -825,6 +844,11 @@ WMPixmap* WMCreateBlendedPixmapFromRImage(WMScreen *scrPtr, RImage *image,
 WMPixmap* WMCreateBlendedPixmapFromFile(WMScreen *scrPtr, const char *fileName,
                                         const RColor *color);
 
+WMPixmap* WMCreateScaledBlendedPixmapFromFile(WMScreen *scrPtr, const char *fileName,
+                                              const RColor *color,
+                                              unsigned int width,
+                                              unsigned int height);
+
 void WMDrawPixmap(WMPixmap *pixmap, Drawable d, int x, int y);
 
 Pixmap WMGetPixmapXID(WMPixmap *pixmap);
@@ -907,6 +931,10 @@ void WMSetWidgetBackgroundColor(WMWidget *w, WMColor *color);
 
 WMColor* WMGetWidgetBackgroundColor(WMWidget *w);
 
+void WMSetWidgetBackgroundPixmap(WMWidget *w, WMPixmap *pix);
+
+WMPixmap *WMGetWidgetBackgroundPixmap(WMWidget *w);
+
 void WMMapSubwidgets(WMWidget *w);
 
 void WMUnmapSubwidgets(WMWidget *w);
@@ -1046,6 +1074,9 @@ void WMSetButtonTextAlignment(WMButton *bPtr, WMAlignment alignment);
 
 void WMSetButtonText(WMButton *bPtr, const char *text);
 
+/* Returns direct pointer to internal data, do not modify! */
+const char *WMGetButtonText(WMButton *bPtr);
+
 void WMSetButtonAltText(WMButton *bPtr, const char *text);
 
 void WMSetButtonTextColor(WMButton *bPtr, WMColor *color);
@@ -1112,6 +1143,8 @@ void WMSetFrameRelief(WMFrame *fPtr, WMReliefType relief);
 
 void WMSetFrameTitle(WMFrame *fPtr, const char *title);
 
+void WMSetFrameTitleColor(WMFrame *fPtr, WMColor *color);
+
 /* ---[ WINGs/wtextfield.c ]---------------------------------------------- */
 
 WMTextField* WMCreateTextField(WMWidget *parent);
@@ -1427,6 +1460,22 @@ void WMSetPopUpButtonEnabled(WMPopUpButton *bPtr, Bool flag);
 
 Bool WMGetPopUpButtonEnabled(WMPopUpButton *bPtr);
 
+/* ---[ WINGs/wprogressindicator.c ]------------------------------------- */
+
+WMProgressIndicator* WMCreateProgressIndicator(WMWidget *parent);
+
+void WMSetProgressIndicatorMinValue(WMProgressIndicator *progressindicator, int value);
+
+void WMSetProgressIndicatorMaxValue(WMProgressIndicator *progressindicator, int value);
+
+void WMSetProgressIndicatorValue(WMProgressIndicator *progressindicator, int value);
+
+int WMGetProgressIndicatorMinValue(WMProgressIndicator *progressindicator);
+
+int WMGetProgressIndicatorMaxValue(WMProgressIndicator *progressindicator);
+
+int WMGetProgressIndicatorValue(WMProgressIndicator *progressindicator);
+
 /* ---[ WINGs/wcolorpanel.c ]--------------------------------------------- */
 
 WMColorPanel* WMGetColorPanel(WMScreen *scrPtr);
@@ -1859,5 +1908,9 @@ void W_setconf_doubleClickDelay(int value);
 }
 #endif /* __cplusplus */
 
-#endif
 
+/* These definitions are not meant to be seen outside this file */
+#undef _wings_noreturn
+
+
+#endif
