@@ -19,13 +19,11 @@
 #define XDND_REQUIRED_TYPES(dragInfo) dragInfo->destInfo->requiredTypes
 #define XDND_SOURCE_ACTION(dragInfo) dragInfo->sourceAction
 #define XDND_DEST_ACTION(dragInfo) dragInfo->destinationAction
-#define XDND_SOURCE_OPERATIONS(dragInfo) dragInfo->destInfo->sourceOperations
 #define XDND_DROP_DATAS(dragInfo) dragInfo->destInfo->dropDatas
-#define XDND_DROP_DATA_COUNT(dragInfo) dragInfo->destInfo->dropDataCount
 #define XDND_DEST_VIEW_IS_REGISTERED(dragInfo) ((dragInfo->destInfo) != NULL)\
     && ((dragInfo->destInfo->destView->dragDestinationProcs) != NULL)
 
-static unsigned char XDNDversion = XDND_VERSION;
+static const unsigned char XDNDversion = XDND_VERSION;
 static WMHandlerID dndDestinationTimer = NULL;
 
 static void *idleState(WMView * destView, XClientMessageEvent * event, WMDraggingInfo * info);
@@ -407,6 +405,12 @@ storeDropData(WMView * destView, Atom selection, Atom target, Time timestamp, vo
 	WMDraggingInfo *info = &(scr->dragInfo);
 	WMData *dataToStore = NULL;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) selection;
+	(void) target;
+	(void) timestamp;
+	(void) cdata;
+
 	if (data != NULL)
 		dataToStore = WMRetainData(data);
 
@@ -523,7 +527,7 @@ static void *checkActionAllowed(WMView * destView, WMDraggingInfo * info)
 	return dropAllowedState;
 }
 
-static void *checkDropAllowed(WMView * destView, XClientMessageEvent * event, WMDraggingInfo * info)
+static void *checkDropAllowed(WMView *destView, WMDraggingInfo *info)
 {
 	storeRequiredTypeList(info);
 
@@ -658,7 +662,7 @@ static void *idleState(WMView * destView, XClientMessageEvent * event, WMDraggin
 
 			if (XDND_SOURCE_TYPES(info) != NULL) {
 				/* enter message infos are available */
-				return checkDropAllowed(destView, event, info);
+				return checkDropAllowed(destView, info);
 			}
 
 			/* waiting for enter message */
@@ -679,7 +683,7 @@ static void *waitEnterState(WMView * destView, XClientMessageEvent * event, WMDr
 
 	if (sourceMsg == scr->xdndEnterAtom) {
 		W_DragDestinationStoreEnterMsgInfo(info, destView, event);
-		return checkDropAllowed(destView, event, info);
+		return checkDropAllowed(destView, info);
 	}
 
 	return waitEnterState;
@@ -722,7 +726,7 @@ static void *dropNotAllowedState(WMView * destView, XClientMessageEvent * event,
 
 	if (sourceMsg == scr->xdndPositionAtom) {
 		if (XDND_SOURCE_ACTION_CHANGED(info)) {
-			return checkDropAllowed(destView, event, info);
+			return checkDropAllowed(destView, info);
 		} else {
 			sendStatusMessage(destView, info, None);
 			return dropNotAllowedState;
@@ -758,7 +762,7 @@ static void *dropAllowedState(WMView * destView, XClientMessageEvent * event, WM
 
 	if (sourceMsg == scr->xdndPositionAtom) {
 		if (XDND_SOURCE_ACTION_CHANGED(info)) {
-			return checkDropAllowed(destView, event, info);
+			return checkDropAllowed(destView, info);
 		} else {
 			sendStatusMessage(destView, info, XDND_DEST_ACTION(info));
 			return dropAllowedState;
@@ -832,7 +836,7 @@ static void realizedObserver(void *self, WMNotification * notif)
 	WMRemoveNotificationObserver(self);
 }
 
-static void W_SetXdndAwareProperty(WMScreen * scr, WMView * view, Atom * types, int typeCount)
+static void W_SetXdndAwareProperty(WMScreen *scr, WMView *view)
 {
 	WMView *toplevel = W_TopLevelOfView(view);
 
@@ -869,7 +873,7 @@ void WMRegisterViewForDraggedTypes(WMView * view, WMArray * acceptedTypes)
 	view->droppableTypes = types;
 	/* WMFreeArray(acceptedTypes); */
 
-	W_SetXdndAwareProperty(W_VIEW_SCREEN(view), view, types, typeCount);
+	W_SetXdndAwareProperty(W_VIEW_SCREEN(view), view);
 }
 
 void WMUnregisterViewDraggedTypes(WMView * view)
@@ -888,6 +892,11 @@ void WMUnregisterViewDraggedTypes(WMView * view)
 static WMDragOperationType
 defAllowedOperation(WMView * self, WMDragOperationType requestedOperation, WMArray * sourceDataTypes)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) self;
+	(void) requestedOperation;
+	(void) sourceDataTypes;
+
 	/* no operation allowed */
 	return WDOperationNone;
 }
@@ -903,6 +912,11 @@ defAllowedOperation(WMView * self, WMDragOperationType requestedOperation, WMArr
 static WMArray *defRequiredDataTypes(WMView * self,
 				     WMDragOperationType requestedOperation, WMArray * sourceDataTypes)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) self;
+	(void) requestedOperation;
+	(void) sourceDataTypes;
+
 	/* no data type allowed (NULL even at 2nd pass) */
 	return NULL;
 }
@@ -912,6 +926,8 @@ static WMArray *defRequiredDataTypes(WMView * self,
  */
 static void defPrepareForDragOperation(WMView * self)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) self;
 }
 
 /*
@@ -938,19 +954,24 @@ static void defPrepareForDragOperation(WMView * self)
 static void
 defPerformDragOperation(WMView * self, WMArray * dropDatas, WMArray * operationList, WMPoint * dropLocation)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) self;
+	(void) dropDatas;
+	(void) operationList;
+	(void) dropLocation;
 }
 
 /* Executed after drop */
 static void defConcludeDragOperation(WMView * self)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) self;
 }
 
 void WMSetViewDragDestinationProcs(WMView * view, WMDragDestinationProcs * procs)
 {
 	if (view->dragDestinationProcs == NULL) {
 		view->dragDestinationProcs = wmalloc(sizeof(WMDragDestinationProcs));
-	} else {
-		free(view->dragDestinationProcs);
 	}
 
 	*view->dragDestinationProcs = *procs;
