@@ -30,8 +30,7 @@
 
 static WMenuParser menu_parser_create_new(const char *file_name, void *file,
 					  const char *include_default_paths);
-static char *menu_parser_isolate_token(WMenuParser parser,
-				       WParserMacro *list_macros);
+static char *menu_parser_isolate_token(WMenuParser parser);
 static void menu_parser_get_directive(WMenuParser parser);
 static Bool menu_parser_include_file(WMenuParser parser);
 static void menu_parser_condition_ifmacro(WMenuParser parser, Bool check_exists);
@@ -179,7 +178,7 @@ read_next_line:
 			goto read_next_line;
 
 		/* Found a word */
-		token = menu_parser_isolate_token(cur_parser, top_parser->macros);
+		token = menu_parser_isolate_token(cur_parser);
 		switch (scanmode) {
 		case GET_TITLE:
 			*title = token;
@@ -299,7 +298,7 @@ found_end_of_comment:
 
 /* read a token (non-spaces suite of characters)
  * the result is wmalloc's, so it needs to be free'd */
-static char *menu_parser_isolate_token(WMenuParser parser, WParserMacro *list_macros)
+static char *menu_parser_isolate_token(WMenuParser parser)
 {
 	char buffer_token[sizeof(parser->line_buffer)];
 	char *token;
@@ -378,7 +377,7 @@ found_end_squote:
 				if (limit-- > 0)
 					goto restart_token_split;
 
-				WMenuParserError(parser, _("too many nested macro expansion, breaking loop") );
+				WMenuParserError(parser, _("too many nested macro expansions, breaking loop"));
 
 				while (isnamechr(*parser->rd))
 					parser->rd++;
@@ -437,7 +436,7 @@ static void menu_parser_get_directive(WMenuParser parser)
 		menu_parser_condition_end(parser);
 
 	} else {
-		WMenuParserError(parser, _("unknow directive '#%s'"), command);
+		WMenuParserError(parser, _("unknown directive '#%s'"), command);
 		return;
 	}
 
@@ -501,7 +500,7 @@ found_end_define_fname:
 			count++;
 
 		if (count > MAX_NESTED_INCLUDES) {
-			WMenuParserError(parser, _("too many nested includes") );
+			WMenuParserError(parser, _("too many nested #include's"));
 			return False;
 		}
 	}
@@ -566,7 +565,7 @@ found_end_define_fname:
 					src++;
 			}
 		}
-		WMenuParserError(parser, _("could not find file \"%s\" for include"), req_filename);
+		WMenuParserError(parser, _("could not find file \"%s\" for #include"), req_filename);
 		return False;
 	}
 
@@ -597,7 +596,7 @@ static void menu_parser_condition_ifmacro(WMenuParser parser, Bool check_exists)
 		parser->rd++;
 
 	/* Add this condition to the stack of conditions */
-	if (parser->cond.depth >= sizeof(parser->cond.stack) / sizeof(parser->cond.stack[0])) {
+	if (parser->cond.depth >= wlengthof(parser->cond.stack)) {
 		WMenuParserError(parser, _("too many nested #if sequences") );
 		return;
 	}
@@ -624,7 +623,7 @@ static void menu_parser_condition_ifmacro(WMenuParser parser, Bool check_exists)
 static void menu_parser_condition_else(WMenuParser parser)
 {
 	if (parser->cond.depth <= 0) {
-		WMenuParserError(parser, _("found #%s but have no matching #if"), "else" );
+		WMenuParserError(parser, _("found #%s but has no matching #if"), "else");
 		return;
 	}
 
@@ -641,7 +640,7 @@ static void menu_parser_condition_end(WMenuParser parser)
 	int idx;
 
 	if (parser->cond.depth <= 0) {
-		WMenuParserError(parser, _("found #%s but have no matching #if"), "endif" );
+		WMenuParserError(parser, _("found #%s but has no matching #if"), "endif");
 		return;
 	}
 
