@@ -97,6 +97,66 @@ static char *RADIO_BUTTON_OFF[] = {
 	".....     .....",
 };
 
+#define TRISTATE_BUTTON_ON_WIDTH 	15
+#define TRISTATE_BUTTON_ON_HEIGHT	15
+static char *TRISTATE_BUTTON_ON[] = {
+	"%%%%%%%%%%%%%%.",
+	"%%%%%%%%%%%%%. ",
+	"%%           . ",
+	"%% ##     ## . ",
+	"%% ###   ### . ",
+	"%%  ### ###  . ",
+	"%%   #####   . ",
+	"%%    ###    . ",
+	"%%   #####   . ",
+	"%%  ### ###  . ",
+	"%% ###   ### . ",
+	"%% ##     ## . ",
+	"%%           . ",
+	"%............. ",
+	".              ",
+};
+
+#define TRISTATE_BUTTON_OFF_WIDTH 	15
+#define TRISTATE_BUTTON_OFF_HEIGHT	15
+static char *TRISTATE_BUTTON_OFF[] = {
+	"%%%%%%%%%%%%%%.",
+	"%%%%%%%%%%%%%. ",
+	"%%           . ",
+	"%%           . ",
+	"%%           . ",
+	"%%           . ",
+	"%%           . ",
+	"%%           . ",
+	"%%           . ",
+	"%%           . ",
+	"%%           . ",
+	"%%           . ",
+	"%%           . ",
+	"%............. ",
+	".              ",
+};
+
+#define TRISTATE_BUTTON_TRI_WIDTH 	15
+#define TRISTATE_BUTTON_TRI_HEIGHT	15
+static char *TRISTATE_BUTTON_TRI[] = {
+	"%%%%%%%%%%%%%%.",
+	"%%%%%%%%%%%%%. ",
+	"%%           . ",
+	"%% # # # # # . ",
+	"%%  # # # #  . ",
+	"%% # # # # # . ",
+	"%%  # # # #  . ",
+	"%% # # # # # . ",
+	"%%  # # # #  . ",
+	"%% # # # # # . ",
+	"%%  # # # #  . ",
+	"%% # # # # # . ",
+	"%%           . ",
+	"%............. ",
+	".              ",
+};
+
 static char *BUTTON_ARROW[] = {
 	"..................",
 	"....##....#### ...",
@@ -363,10 +423,7 @@ static WMPixmap *makePixmap(W_Screen * sPtr, char **data, int width, int height,
 }
 
 #define T_WINGS_IMAGES_FILE  RESOURCE_PATH"/Images.tiff"
-#define T_DEFAULT_OBJECT_ICON_FILE RESOURCE_PATH"/defaultIcon.tiff"
-
 #define X_WINGS_IMAGES_FILE  RESOURCE_PATH"/Images.xpm"
-#define X_DEFAULT_OBJECT_ICON_FILE RESOURCE_PATH"/defaultIcon.xpm"
 
 static Bool loadPixmaps(WMScreen * scr)
 {
@@ -474,16 +531,6 @@ static Bool loadPixmaps(WMScreen * scr)
 
 	RReleaseImage(image);
 
-#if 0
-	scr->defaultObjectIcon = WMCreatePixmapFromFile(scr, T_DEFAULT_OBJECT_ICON_FILE);
-	if (!scr->defaultObjectIcon) {
-		scr->defaultObjectIcon = WMCreatePixmapFromFile(scr, X_DEFAULT_OBJECT_ICON_FILE);
-	}
-	if (!scr->defaultObjectIcon) {
-		wwarning("WINGs: could not load default icon file");
-		return False;
-	}
-#endif
 	return True;
 }
 
@@ -492,7 +539,7 @@ WMScreen *WMOpenScreen(const char *display)
 	Display *dpy = XOpenDisplay(display);
 
 	if (!dpy) {
-		wwarning("WINGs: could not open display %s", XDisplayName(display));
+		wwarning(_("WINGs: could not open display %s"), XDisplayName(display));
 		return NULL;
 	}
 
@@ -550,7 +597,7 @@ WMScreen *WMCreateScreenWithRContext(Display * display, int screen, RContext * c
 		"_NET_WM_ICON_NAME",
 		"_NET_WM_ICON",
 	};
-	Atom atoms[sizeof(atomNames) / sizeof(atomNames[0])];
+	Atom atoms[wlengthof(atomNames)];
 	int i;
 
 	if (!initialized) {
@@ -742,6 +789,15 @@ WMScreen *WMCreateScreenWithRContext(Display * display, int screen, RContext * c
 	scrPtr->radioButtonImageOff = makePixmap(scrPtr, RADIO_BUTTON_OFF,
 						 RADIO_BUTTON_OFF_WIDTH, RADIO_BUTTON_OFF_HEIGHT, False);
 
+	scrPtr->tristateButtonImageOn = makePixmap(scrPtr, TRISTATE_BUTTON_ON,
+	                                           TRISTATE_BUTTON_ON_WIDTH, TRISTATE_BUTTON_ON_HEIGHT, False);
+
+	scrPtr->tristateButtonImageOff = makePixmap(scrPtr, TRISTATE_BUTTON_OFF,
+	                                            TRISTATE_BUTTON_OFF_WIDTH, TRISTATE_BUTTON_OFF_HEIGHT, False);
+
+	scrPtr->tristateButtonImageTri = makePixmap(scrPtr, TRISTATE_BUTTON_TRI,
+	                                            TRISTATE_BUTTON_TRI_WIDTH, TRISTATE_BUTTON_TRI_HEIGHT, False);
+
 	scrPtr->buttonArrow = makePixmap(scrPtr, BUTTON_ARROW, BUTTON_ARROW_WIDTH, BUTTON_ARROW_HEIGHT, False);
 
 	scrPtr->pushedButtonArrow = makePixmap(scrPtr, BUTTON_ARROW2,
@@ -801,9 +857,9 @@ WMScreen *WMCreateScreenWithRContext(Display * display, int screen, RContext * c
 	}
 
 #ifdef HAVE_XINTERNATOMS
-	XInternAtoms(display, atomNames, sizeof(atomNames) / sizeof(atomNames[0]), False, atoms);
+	XInternAtoms(display, atomNames, wlengthof(atomNames), False, atoms);
 #else
-	for (i = 0; i < sizeof(atomNames) / sizeof(atomNames[0]); i++) {
+	for (i = 0; i < wlengthof(atomNames); i++) {
 		atoms[i] = XInternAtom(display, atomNames[i], False);
 	}
 #endif
@@ -960,6 +1016,21 @@ WMColor *WMGetWidgetBackgroundColor(WMWidget * w)
 	return W_VIEW(w)->backColor;
 }
 
+void WMSetWidgetBackgroundPixmap(WMWidget *w, WMPixmap *pix)
+{
+	if (!pix)
+		return;
+
+	W_SetViewBackgroundPixmap(W_VIEW(w), pix);
+	if (W_VIEW(w)->flags.mapped)
+		WMRedisplayWidget(w);
+}
+
+WMPixmap *WMGetWidgetBackgroundPixmap(WMWidget *w)
+{
+	return W_VIEW(w)->backImage;
+}
+
 void WMRaiseWidget(WMWidget * w)
 {
 	W_RaiseView(W_VIEW(w));
