@@ -49,6 +49,9 @@ static void makeKnobPixmap(Slider * sPtr);
 
 static void realizeObserver(void *self, WMNotification * not)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) not;
+
 	makeKnobPixmap(self);
 }
 
@@ -255,6 +258,9 @@ static void didResizeSlider(W_ViewDelegate * self, WMView * view)
 	int width = sPtr->view->size.width;
 	int height = sPtr->view->size.height;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) self;
+
 	assert(width > 0);
 	assert(height > 0);
 
@@ -404,16 +410,16 @@ static void handleActionEvents(XEvent * event, void *data)
 
 	switch (event->type) {
 	case ButtonPress:
-		if (event->xbutton.button == WINGsConfiguration.mouseWheelUp && !sPtr->flags.dragging) {
-			/* Wheel up */
+		if (event->xbutton.button == WINGsConfiguration.mouseWheelDown && !sPtr->flags.dragging) {
+			/* Wheel down */
 			if (sPtr->value + 1 <= sPtr->maxValue) {
 				WMSetSliderValue(sPtr, sPtr->value + 1);
 				if (sPtr->flags.continuous && sPtr->action) {
 					(*sPtr->action) (sPtr, sPtr->clientData);
 				}
 			}
-		} else if (event->xbutton.button == WINGsConfiguration.mouseWheelDown && !sPtr->flags.dragging) {
-			/* Wheel down */
+		} else if (event->xbutton.button == WINGsConfiguration.mouseWheelUp && !sPtr->flags.dragging) {
+			/* Wheel up */
 			if (sPtr->value - 1 >= sPtr->minValue) {
 				WMSetSliderValue(sPtr, sPtr->value - 1);
 				if (sPtr->flags.continuous && sPtr->action) {
