@@ -581,7 +581,7 @@ static Bool updateStartForCurrentTextBlock(Text * tPtr, int x, int y, int *dir,
 			return 0;
 
 		if (tb->graphic) {
-			tPtr->currentTextBlock = (dir ? tPtr->lastTextBlock : tPtr->firstTextBlock);
+			tPtr->currentTextBlock = (*dir ? tPtr->lastTextBlock : tPtr->firstTextBlock);
 			tPtr->tpos = 0;
 			return 0;
 		}
@@ -1023,7 +1023,6 @@ static void cursorToTextPosition(Text * tPtr, int x, int y)
 			if ((dir ? tb->next : tb->prior)) {
 				tb = (dir ? tb->next : tb->prior);
 			} else {
-				pos = tb->used;
 				break;	/* goto _doneH; */
 			}
 		}
@@ -1048,9 +1047,6 @@ static void cursorToTextPosition(Text * tPtr, int x, int y)
 		}
 	}
 
-	if (tb->blank)
-		_w = 0;
-
 	_y = tb->sections[s]._y;
 
 	while (tb) {
@@ -1117,7 +1113,6 @@ static void cursorToTextPosition(Text * tPtr, int x, int y)
 		if ((dir ? tb->next : tb->prior)) {
 			tb = (dir ? tb->next : tb->prior);
 		} else {
-			done = True;
 			break;
 		}
 
@@ -1253,7 +1248,6 @@ static void scrollersCallBack(WMWidget * w, void *self)
 					tPtr->vpos -= 16;
 				else
 					tPtr->vpos = 0;
-				scroll = True;
 			}
 			break;
 
@@ -1264,7 +1258,6 @@ static void scrollersCallBack(WMWidget * w, void *self)
 						tPtr->vpos += 16;
 					else
 						tPtr->vpos = limit;
-					scroll = True;
 				}
 			}
 			break;
@@ -1274,21 +1267,17 @@ static void scrollersCallBack(WMWidget * w, void *self)
 				tPtr->vpos -= height;
 			else
 				tPtr->vpos = 0;
-
-			scroll = True;
 			break;
 
 		case WSIncrementPage:
 			tPtr->vpos += height;
 			if (tPtr->vpos > (tPtr->docHeight - height))
 				tPtr->vpos = tPtr->docHeight - height;
-			scroll = True;
 			break;
 
 		case WSKnob:
 			tPtr->vpos = WMGetScrollerValue(tPtr->vS)
 			    * (float)(tPtr->docHeight - height);
-			scroll = True;
 			break;
 
 		case WSKnobSlot:
@@ -1311,7 +1300,6 @@ static void scrollersCallBack(WMWidget * w, void *self)
 					tPtr->hpos -= 16;
 				else
 					tPtr->hpos = 0;
-				scroll = True;
 			}
 			break;
 
@@ -1322,7 +1310,6 @@ static void scrollersCallBack(WMWidget * w, void *self)
 						tPtr->hpos += 16;
 					else
 						tPtr->hpos = limit;
-					scroll = True;
 				}
 			}
 			break;
@@ -1332,21 +1319,17 @@ static void scrollersCallBack(WMWidget * w, void *self)
 				tPtr->hpos -= width;
 			else
 				tPtr->hpos = 0;
-
-			scroll = True;
 			break;
 
 		case WSIncrementPage:
 			tPtr->hpos += width;
 			if (tPtr->hpos > (tPtr->docWidth - width))
 				tPtr->hpos = tPtr->docWidth - width;
-			scroll = True;
 			break;
 
 		case WSKnob:
 			tPtr->hpos = WMGetScrollerValue(tPtr->hS)
 			    * (float)(tPtr->docWidth - width);
-			scroll = True;
 			break;
 
 		case WSKnobSlot:
@@ -1697,6 +1680,9 @@ static void textDidResize(W_ViewDelegate * self, WMView * view)
 	unsigned short h = tPtr->view->size.height;
 	unsigned short rh = 0, vw = 0, rel;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) self;
+
 	rel = (tPtr->flags.relief == WRFlat);
 
 	if (tPtr->ruler && tPtr->flags.rulerShown) {
@@ -2056,12 +2042,17 @@ static WMData *requestHandler(WMView * view, Atom selection, Atom target, void *
 	Atom COMPOUND_TEXT = XInternAtom(dpy, "COMPOUND_TEXT", False);
 	WMData *data = NULL;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) selection;
+	(void) cdata;
+
 	if (target == XA_STRING || target == TEXT || target == COMPOUND_TEXT) {
 		char *text = WMGetTextSelectedStream(tPtr);
 
 		if (text) {
 			data = WMCreateDataWithBytes(text, strlen(text));
 			WMSetDataFormat(data, TYPETEXT);
+			wfree(text);
 		}
 		*type = target;
 		return data;
@@ -2070,15 +2061,14 @@ static WMData *requestHandler(WMView * view, Atom selection, Atom target, void *
 
 	_TARGETS = XInternAtom(dpy, "TARGETS", False);
 	if (target == _TARGETS) {
-		Atom *ptr;
+		Atom array[4];
 
-		ptr = wmalloc(4 * sizeof(Atom));
-		ptr[0] = _TARGETS;
-		ptr[1] = XA_STRING;
-		ptr[2] = TEXT;
-		ptr[3] = COMPOUND_TEXT;
+		array[0] = _TARGETS;
+		array[1] = XA_STRING;
+		array[2] = TEXT;
+		array[3] = COMPOUND_TEXT;
 
-		data = WMCreateDataWithBytes(ptr, 4 * 4);
+		data = WMCreateDataWithBytes(&array, sizeof(array));
 		WMSetDataFormat(data, 32);
 
 		*type = target;
@@ -2090,6 +2080,10 @@ static WMData *requestHandler(WMView * view, Atom selection, Atom target, void *
 
 static void lostHandler(WMView * view, Atom selection, void *cdata)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) selection;
+	(void) cdata;
+
 	releaseSelection((WMText *) view->self);
 }
 
@@ -2156,8 +2150,14 @@ static void autoSelectText(Text * tPtr, int clicks)
 	} else if (clicks == 3) {
 		TextBlock *cur = tb;
 
-		while (tb && !tb->first) {
+		while (!tb->first) {
 			tb = tb->prior;
+			if (tb == NULL) {
+#ifndef NDEBUG
+				wwarning("corrupted list of text blocks in WMText, while searching for first block");
+#endif
+				goto error_select_3clicks;
+			}
 		}
 		tPtr->sel.y = tb->sections[0]._y;
 
@@ -2167,6 +2167,7 @@ static void autoSelectText(Text * tPtr, int clicks)
 		}
 		tPtr->sel.h = tb->sections[tb->nsections - 1]._y + 5 - tPtr->sel.y;
 
+	error_select_3clicks:
 		tPtr->sel.x = 0;
 		tPtr->sel.w = tPtr->docWidth;
 		tPtr->clicked.x = 0;	/* only for now, fix sel. code */
@@ -2229,11 +2230,8 @@ static void handleTextKeyPress(Text * tPtr, XEvent * event)
 	case XK_Left:
 		if (!(tb = tPtr->currentTextBlock))
 			break;
-		if (tb->graphic)
-			goto L_imaGFX;
 
-		if (tPtr->tpos == 0) {
- L_imaGFX:
+		if (tb->graphic || tPtr->tpos == 0) {
 			if (tb->prior) {
 				tPtr->currentTextBlock = tb->prior;
 				if (tPtr->currentTextBlock->graphic)
@@ -2254,10 +2252,7 @@ static void handleTextKeyPress(Text * tPtr, XEvent * event)
 	case XK_Right:
 		if (!(tb = tPtr->currentTextBlock))
 			break;
-		if (tb->graphic)
-			goto R_imaGFX;
-		if (tPtr->tpos == tb->used) {
- R_imaGFX:
+		if (tb->graphic || tPtr->tpos == tb->used) {
 			if (tb->next) {
 				tPtr->currentTextBlock = tb->next;
 				tPtr->tpos = 0;
@@ -2337,6 +2332,12 @@ static void pasteText(WMView * view, Atom selection, Atom target, Time timestamp
 	Text *tPtr = (Text *) view->self;
 	char *text;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) selection;
+	(void) target;
+	(void) timestamp;
+	(void) cdata;
+
 	tPtr->flags.waitingForSelection = 0;
 
 	if (data) {
@@ -2675,6 +2676,9 @@ static void rulerMoveCallBack(WMWidget * w, void *self)
 {
 	Text *tPtr = (Text *) self;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) w;
+
 	if (!tPtr)
 		return;
 	if (W_CLASS(tPtr) != WC_Text)
@@ -2687,6 +2691,9 @@ static void rulerReleaseCallBack(WMWidget * w, void *self)
 {
 	Text *tPtr = (Text *) self;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) w;
+
 	if (!tPtr)
 		return;
 	if (W_CLASS(tPtr) != WC_Text)
@@ -2703,11 +2710,17 @@ static WMArray *dropDataTypes(WMView * self)
 
 static WMDragOperationType wantedDropOperation(WMView * self)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) self;
+
 	return WDOperationCopy;
 }
 
 static Bool acceptDropOperation(WMView * self, WMDragOperationType allowedOperation)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) self;
+
 	return (allowedOperation == WDOperationCopy);
 }
 
@@ -2746,11 +2759,20 @@ static WMDragSourceProcs _DragSourceProcs = {
 
 static WMArray *requiredDataTypes(WMView * self, WMDragOperationType request, WMArray * sourceDataTypes)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) request;
+	(void) sourceDataTypes;
+
 	return ((Text *) self->self)->xdndDestinationTypes;
 }
 
 static WMDragOperationType allowedOperation(WMView * self, WMDragOperationType request, WMArray * sourceDataTypes)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) self;
+	(void) request;
+	(void) sourceDataTypes;
+
 	return WDOperationCopy;
 }
 
@@ -2761,6 +2783,10 @@ static void performDragOperation(WMView * self, WMArray * dropData, WMArray * op
 	char *colorName;
 	WMColor *color;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) operations;
+	(void) dropLocation;
+
 	if (tPtr) {
 
 		/* only one required type, implies only one drop data */
@@ -2866,7 +2892,7 @@ static void releaseStreamObjects(void *data)
 
 static WMArray *getStreamObjects(WMText * tPtr, int sel)
 {
-	WMArray *array = WMCreateArrayWithDestructor(4, releaseStreamObjects);
+	WMArray *array;
 	WMData *data;
 	char *stream;
 	unsigned short len;
@@ -2876,6 +2902,8 @@ static WMArray *getStreamObjects(WMText * tPtr, int sel)
 	if (!stream)
 		return NULL;
 
+	array = WMCreateArrayWithDestructor(4, releaseStreamObjects);
+
 	start = stream;
 	while (start) {
 
@@ -2994,6 +3022,7 @@ WMText *WMCreateTextForDocumentType(WMWidget * parent, WMAction * parser, WMActi
 		WMAddToArray(types, "application/X-color");
 		WMAddToArray(types, "application/X-image");
 		WMRegisterViewForDraggedTypes(tPtr->view, types);
+		WMFreeArray(types);
 	}
 
 	/*WMAddNotificationObserver(fontChanged, tPtr,
@@ -3274,7 +3303,7 @@ WMSetTextBlockProperties(WMText * tPtr, void *vtb, unsigned int first,
 
 void
 WMGetTextBlockProperties(WMText * tPtr, void *vtb, unsigned int *first,
-			 unsigned int *kanji, unsigned int *underlined, int *script, WMRulerMargins * margins)
+			 unsigned int *kanji, unsigned int *underlined, int *script, WMRulerMargins *margins)
 {
 	TextBlock *tb = (TextBlock *) vtb;
 	if (!tb)
@@ -3289,16 +3318,11 @@ WMGetTextBlockProperties(WMText * tPtr, void *vtb, unsigned int *first,
 	if (script)
 		*script = tb->script;
 	if (margins)
-		margins = &tPtr->margins[tb->marginN];
+		*margins = tPtr->margins[tb->marginN];
 }
 
-void WMPrependTextBlock(WMText * tPtr, void *vtb)
+static int prepareTextBlock(WMText *tPtr, TextBlock *tb)
 {
-	TextBlock *tb = (TextBlock *) vtb;
-
-	if (!tb)
-		return;
-
 	if (tb->graphic) {
 		if (tb->object) {
 			WMWidget *w = tb->d.widget;
@@ -3318,13 +3342,24 @@ void WMPrependTextBlock(WMText * tPtr, void *vtb)
 		tb->next = tb->prior = NULL;
 		tb->first = True;
 		tPtr->lastTextBlock = tPtr->firstTextBlock = tPtr->currentTextBlock = tb;
-		return;
+		return 0;
 	}
 
 	if (!tb->first) {
 		tb->marginN = tPtr->currentTextBlock->marginN;
 	}
 
+	return 1;
+}
+
+
+void WMPrependTextBlock(WMText *tPtr, void *vtb)
+{
+	TextBlock *tb = (TextBlock *) vtb;
+
+	if (!tb || !prepareTextBlock(tPtr, tb))
+		return;
+
 	tb->next = tPtr->currentTextBlock;
 	tb->prior = tPtr->currentTextBlock->prior;
 	if (tPtr->currentTextBlock->prior)
@@ -3337,39 +3372,13 @@ void WMPrependTextBlock(WMText * tPtr, void *vtb)
 	tPtr->currentTextBlock = tb;
 }
 
-void WMAppendTextBlock(WMText * tPtr, void *vtb)
+void WMAppendTextBlock(WMText *tPtr, void *vtb)
 {
 	TextBlock *tb = (TextBlock *) vtb;
 
-	if (!tb)
+	if (!tb || !prepareTextBlock(tPtr, tb))
 		return;
 
-	if (tb->graphic) {
-		if (tb->object) {
-			WMWidget *w = tb->d.widget;
-			if (W_CLASS(w) != WC_TextField && W_CLASS(w) != WC_Text) {
-				(W_VIEW(w))->attribs.cursor = tPtr->view->screen->defaultCursor;
-				(W_VIEW(w))->attribFlags |= CWOverrideRedirect | CWCursor;
-			}
-		}
-		WMAddToArray(tPtr->gfxItems, (void *)tb);
-		tPtr->tpos = 1;
-
-	} else {
-		tPtr->tpos = tb->used;
-	}
-
-	if (!tPtr->lastTextBlock || !tPtr->firstTextBlock) {
-		tb->next = tb->prior = NULL;
-		tb->first = True;
-		tPtr->lastTextBlock = tPtr->firstTextBlock = tPtr->currentTextBlock = tb;
-		return;
-	}
-
-	if (!tb->first) {
-		tb->marginN = tPtr->currentTextBlock->marginN;
-	}
-
 	tb->next = tPtr->currentTextBlock->next;
 	tb->prior = tPtr->currentTextBlock;
 	if (tPtr->currentTextBlock->next)
@@ -3431,6 +3440,10 @@ static void destroyWidget(WMWidget * widget)
 void WMDestroyTextBlock(WMText * tPtr, void *vtb)
 {
 	TextBlock *tb = (TextBlock *) vtb;
+
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) tPtr;
+
 	if (!tb)
 		return;
 
