@@ -91,20 +91,21 @@ typedef struct _Panel {
 
 #define ICON_FILE	"fonts"
 
-static struct {
-	char *option;
-	char *label;
+static const struct {
+	const char *option;
+	const char *label;
 } fontOptions[] = {
-	{
-	"WindowTitleFont", N_("Window Title")}, {
-	"MenuTitleFont", N_("Menu Title")}, {
-	"MenuTextFont", N_("Menu Text")}, {
-	"IconTitleFont", N_("Icon Title")}, {
-	"ClipTitleFont", N_("Clip Title")}, {
-	"LargeDisplayFont", N_("Desktop Caption")}, {
-NULL, NULL},};
+	{ "WindowTitleFont", N_("Window Title") },
+	{ "MenuTitleFont", N_("Menu Title") },
+	{ "MenuTextFont", N_("Menu Text") },
+	{ "IconTitleFont", N_("Icon Title") },
+	{ "ClipTitleFont", N_("Clip Title") },
+	{ "LargeDisplayFont", N_("Desktop Caption") },
+	{ "SystemFont", N_("System Font") },
+	{ "BoldSystemFont", N_("Bold System Font")}
+};
 
-static char *standardSizes[] = {
+static const char *standardSizes[] = {
 	"6",
 	"8",
 	"9",
@@ -128,52 +129,43 @@ static char *standardSizes[] = {
 	NULL
 };
 
-static struct {
+static const struct {
 	int weight;
-	char *name;
+	const char *name;
 } fontWeights[] = {
-	{
-	FC_WEIGHT_THIN, "Thin"}, {
-	FC_WEIGHT_EXTRALIGHT, "ExtraLight"}, {
-	FC_WEIGHT_LIGHT, "Light"}, {
-	FC_WEIGHT_NORMAL, "Normal"}, {
-	FC_WEIGHT_MEDIUM, ""},	/*"medium"}, */
-	{
-	FC_WEIGHT_DEMIBOLD, "DemiBold"}, {
-	FC_WEIGHT_BOLD, "Bold"}, {
-	FC_WEIGHT_EXTRABOLD, "ExtraBold"}, {
-	FC_WEIGHT_BLACK, "Black"}, {
-	0, NULL}
+	{ FC_WEIGHT_THIN, "Thin" },
+	{ FC_WEIGHT_EXTRALIGHT, "ExtraLight" },
+	{ FC_WEIGHT_LIGHT, "Light" },
+	{ FC_WEIGHT_NORMAL, "Normal" },
+	{ FC_WEIGHT_MEDIUM, "" },
+	{ FC_WEIGHT_DEMIBOLD, "DemiBold" },
+	{ FC_WEIGHT_BOLD, "Bold" },
+	{ FC_WEIGHT_EXTRABOLD, "ExtraBold" },
+	{ FC_WEIGHT_BLACK, "Black" }
 };
 
-static struct {
+static const struct {
 	int slant;
-	char *name;
+	const char *name;
 } fontSlants[] = {
-	{
-	FC_SLANT_ROMAN, ""},	/*"Roman"}, */
-	{
-	FC_SLANT_ITALIC, "Italic"}, {
-	FC_SLANT_OBLIQUE, "Oblique"}, {
-	0, NULL}
+	{ FC_SLANT_ROMAN, "" },
+	{ FC_SLANT_ITALIC, "Italic" },
+	{ FC_SLANT_OBLIQUE, "Oblique" }
 };
 
-static struct {
+static const struct {
 	int width;
-	char *name;
+	const char *name;
 } fontWidths[] = {
-	{
-	FC_WIDTH_ULTRACONDENSED, "UltraCondensed"}, {
-	FC_WIDTH_EXTRACONDENSED, "ExtraCondensed"}, {
-	FC_WIDTH_CONDENSED, "Condensed"}, {
-	FC_WIDTH_SEMICONDENSED, "SemiCondensed"}, {
-	FC_WIDTH_NORMAL, ""},	/*"normal"}, */
-	{
-	FC_WIDTH_SEMIEXPANDED, "SemiExpanded"}, {
-	FC_WIDTH_EXPANDED, "Expanded"}, {
-	FC_WIDTH_EXTRAEXPANDED, "ExtraExpanded"}, {
-	FC_WIDTH_ULTRAEXPANDED, "UltraExpanded"}, {
-	0, NULL}
+	{ FC_WIDTH_ULTRACONDENSED, "UltraCondensed" },
+	{ FC_WIDTH_EXTRACONDENSED, "ExtraCondensed" },
+	{ FC_WIDTH_CONDENSED, "Condensed" },
+	{ FC_WIDTH_SEMICONDENSED, "SemiCondensed" },
+	{ FC_WIDTH_NORMAL, "" },
+	{ FC_WIDTH_SEMIEXPANDED, "SemiExpanded" },
+	{ FC_WIDTH_EXPANDED, "Expanded" },
+	{ FC_WIDTH_EXTRAEXPANDED, "ExtraExpanded" },
+	{ FC_WIDTH_ULTRAEXPANDED, "UltraExpanded" },
 };
 
 static int compare_family(const void *a, const void *b)
@@ -355,6 +347,9 @@ static void selectedFamily(WMWidget * w, void *data)
 	FontStyle *oldStyle = NULL;
 	char buffer[1024];
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) w;
+
 	item = WMGetListSelectedItem(panel->styleL);
 	if (item)
 		oldStyle = (FontStyle *) item->clientData;
@@ -368,20 +363,20 @@ static void selectedFamily(WMWidget * w, void *data)
 		WMClearList(panel->styleL);
 		for (i = 0; i < family->stylen; i++) {
 			int j;
-			char *weight = "", *slant = "", *width = "";
+			const char *weight = "", *slant = "", *width = "";
 			WMListItem *item;
 
-			for (j = 0; fontWeights[j].name; j++)
+			for (j = 0; j < wlengthof(fontWeights); j++)
 				if (fontWeights[j].weight == family->styles[i].weight) {
 					weight = fontWeights[j].name;
 					break;
 				}
-			for (j = 0; fontWidths[j].name; j++)
+			for (j = 0; j < wlengthof(fontWidths); j++)
 				if (fontWidths[j].width == family->styles[i].width) {
 					width = fontWidths[j].name;
 					break;
 				}
-			for (j = 0; fontSlants[j].name; j++)
+			for (j = 0; j < wlengthof(fontSlants); j++)
 				if (fontSlants[j].slant == family->styles[i].slant) {
 					slant = fontSlants[j].name;
 					break;
@@ -436,6 +431,9 @@ static void selected(WMWidget * w, void *data)
 	FcChar8 *ofont;
 	char *nfont;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) w;
+
 	ofont = (FcChar8 *) WMGetMenuItemRepresentedObject(item);
 	nfont = getSelectedFont(panel, ofont);
 	if (ofont)
@@ -453,6 +451,9 @@ static void selectedOption(WMWidget * w, void *data)
 	WMMenuItem *item = WMGetPopUpButtonMenuItem(panel->optionP, index);
 	char *font;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) w;
+
 	font = (char *)WMGetMenuItemRepresentedObject(item);
 	if (font) {
 		FcPattern *pat;
@@ -541,7 +542,7 @@ static void selectedOption(WMWidget * w, void *data)
 	updateSampleFont(panel);
 }
 
-static WMLabel *createListLabel(WMScreen * scr, WMWidget * parent, char *text)
+static WMLabel *createListLabel(WMScreen * scr, WMWidget * parent, const char *text)
 {
 	WMLabel *label;
 	WMColor *color;
@@ -578,7 +579,19 @@ static void showData(_Panel * panel)
 		if (ofont)
 			wfree(ofont);
 
-		font = GetStringForKey(fontOptions[i].option);
+		if (strcmp(fontOptions[i].option, "SystemFont") == 0 ||
+		    strcmp(fontOptions[i].option, "BoldSystemFont") == 0) {
+			char *path;
+			WMUserDefaults *defaults;
+
+			path = wdefaultspathfordomain("WMGLOBAL");
+			defaults = WMGetDefaultsFromPath(path);
+			wfree(path);
+			font = WMGetUDStringForKey(defaults,
+						   fontOptions[i].option);
+		} else {
+			font = GetStringForKey(fontOptions[i].option);
+		}
 		if (font)
 			font = wstrdup(font);
 		WMSetMenuItemRepresentedObject(item, font);
@@ -599,7 +612,21 @@ static void storeData(_Panel * panel)
 
 		font = WMGetMenuItemRepresentedObject(item);
 		if (font && *font) {
-			SetStringForKey(font, fontOptions[i].option);
+			if (strcmp(fontOptions[i].option, "SystemFont") == 0 ||
+			    strcmp(fontOptions[i].option, "BoldSystemFont") == 0) {
+				char *path;
+				WMUserDefaults *defaults;
+
+				path = wdefaultspathfordomain("WMGLOBAL");
+				defaults = WMGetDefaultsFromPath(path);
+				wfree(path);
+				WMSetUDStringForKey(defaults,
+						    font,
+						    fontOptions[i].option);
+				WMSaveUserDefaults(defaults);
+			} else {
+				SetStringForKey(font, fontOptions[i].option);
+			}
 		}
 	}
 }
@@ -629,9 +656,8 @@ static void createPanel(Panel * p)
 	WMSetBoxHorizontal(vbox, False);
 	panel->optionP = WMCreatePopUpButton(vbox);
 	WMAddBoxSubviewAtEnd(vbox, WMWidgetView(panel->optionP), False, True, 20, 0, 8);
-	for (i = 0; fontOptions[i].option; i++) {
+	for (i = 0; i < wlengthof(fontOptions); i++)
 		WMAddPopUpButtonItem(panel->optionP, _(fontOptions[i].label));
-	}
 	WMSetPopUpButtonAction(panel->optionP, selectedOption, panel);
 
 	label = WMCreateLabel(hbox);
@@ -707,7 +733,7 @@ static void createPanel(Panel * p)
 	showData(panel);
 }
 
-Panel *InitFontSimple(WMScreen * scr, WMWidget * parent)
+Panel *InitFontSimple(WMWidget *parent)
 {
 	_Panel *panel;
 
