@@ -216,6 +216,10 @@ void wSetFocusTo(WScreen *scr, WWindow *wwin)
 			if (wPreferences.highlight_active_app)
 				wApplicationDeactivate(oapp);
 		}
+
+		/* reset fullscreen if temporarily removed due to lost focus*/
+		if (wwin->flags.fullscreen)
+			ChangeStackingLevel(wwin->frame->core, WMFullscreenLevel);
 	}
 
 	wWindowFocus(wwin, focused);
@@ -225,6 +229,7 @@ void wSetFocusTo(WScreen *scr, WWindow *wwin)
 		wUserMenuRefreshInstances(napp->menu, wwin);
 #endif	/* USER_MENU */
 
+		/* kix: Only menu map with mouse, not alt+tab! */
 		if (wwin->flags.mapped)
 			wAppMenuMap(napp->menu, wwin);
 	}
@@ -354,7 +359,7 @@ void update_saved_geometry(WWindow *wwin)
 		save_old_geometry(wwin, SAVE_GEOMETRY_X);
 }
 
-void wMaximizeWindow(WWindow *wwin, int directions)
+void wMaximizeWindow(WWindow *wwin, int directions, int head)
 {
 	unsigned int new_width, new_height, half_scr_width, half_scr_height;
 	int new_x = 0;
@@ -388,20 +393,15 @@ void wMaximizeWindow(WWindow *wwin, int directions)
 	totalArea.y2 = scr->scr_height;
 	totalArea.x1 = 0;
 	totalArea.y1 = 0;
-	usableArea = totalArea;
 
-	if (!(directions & MAX_IGNORE_XINERAMA)) {
+	/* In case of mouse initiated maximize, use the head in which pointer is
+	   located, rather than window position, which is passed to the function */
+	if (!(directions & MAX_IGNORE_XINERAMA) && !(directions & MAX_KEYBOARD)) {
 		WScreen *scr = wwin->screen_ptr;
-		int head;
-
-		if (directions & MAX_KEYBOARD)
-			head = wGetHeadForWindow(wwin);
-		else
-			head = wGetHeadForPointerLocation(scr);
-
-		usableArea = wGetUsableAreaForHead(scr, head, &totalArea, True);
+		head = wGetHeadForPointerLocation(scr);
 	}
 
+	usableArea = wGetUsableAreaForHead(scr, head, &totalArea, True);
 
 	/* Only save directions, not kbd or xinerama hints */
 	directions &= (MAX_HORIZONTAL | MAX_VERTICAL | MAX_LEFTHALF | MAX_RIGHTHALF | MAX_TOPHALF | MAX_BOTTOMHALF | MAX_MAXIMUS);
@@ -497,20 +497,133 @@ void handleMaximize(WWindow *wwin, int directions)
 	int requested = directions & (MAX_HORIZONTAL | MAX_VERTICAL | MAX_LEFTHALF | MAX_RIGHTHALF | MAX_TOPHALF | MAX_BOTTOMHALF | MAX_MAXIMUS);
 	int effective = requested ^ current;
 	int flags = directions & ~requested;
+	int head = wGetHeadForWindow(wwin);
+	int dest_head = -1;
 
 	if (!effective) {
 		/* allow wMaximizeWindow to restore the Maximusized size */
 		if ((wwin->flags.old_maximized & MAX_MAXIMUS) &&
 				!(requested & MAX_MAXIMUS))
-			wMaximizeWindow(wwin, MAX_MAXIMUS | flags);
-		else
+			wMaximizeWindow(wwin, MAX_MAXIMUS | flags, head);
+
+		else if (wPreferences.alt_half_maximize &&
+				current & MAX_HORIZONTAL && current & MAX_VERTICAL &&
+				requested & MAX_HORIZONTAL && requested & MAX_VERTICAL)
 			wUnmaximizeWindow(wwin);
+
+		/* Apply for window state, which is only horizontally or vertically
+		 * maximized. Quarters cannot be handled here, since there is not clear
+		 * on which direction user intend to move such window. */
+		else if (wPreferences.move_half_max_between_heads &&
+				current & (MAX_VERTICAL | MAX_HORIZONTAL)) {
+			if (requested & MAX_LEFTHALF && current & MAX_LEFTHALF) {
+				dest_head = wGetHeadRelativeToCurrentHead(wwin->screen_ptr,
+						head, DIRECTION_LEFT);
+				if (dest_head != -1) {
+					effective |= MAX_RIGHTHALF;
+					effective |= MAX_VERTICAL;
+					effective &= ~(MAX_HORIZONTAL | MAX_LEFTHALF);
+				}
+			} else if (requested & MAX_RIGHTHALF &&
+					current & MAX_RIGHTHALF) {
+				dest_head = wGetHeadRelativeToCurrentHead(wwin->screen_ptr,
+						head, DIRECTION_RIGHT);
+				if (dest_head != -1) {
+					effective |= MAX_LEFTHALF;
+					effective |= MAX_VERTICAL;
+					effective &= ~(MAX_HORIZONTAL | MAX_RIGHTHALF);
+				}
+			} else if (requested & MAX_TOPHALF && current & MAX_TOPHALF) {
+				dest_head = wGetHeadRelativeToCurrentHead(wwin->screen_ptr,
+						head, DIRECTION_UP);
+				if (dest_head != -1) {
+					effective |= MAX_BOTTOMHALF;
+					effective |= MAX_HORIZONTAL;
+					effective &= ~(MAX_VERTICAL | MAX_TOPHALF);
+				}
+			} else if (requested & MAX_BOTTOMHALF &&
+					current & MAX_BOTTOMHALF) {
+				dest_head = wGetHeadRelativeToCurrentHead(wwin->screen_ptr,
+						head, DIRECTION_DOWN);
+				if (dest_head != -1) {
+					effective |= MAX_TOPHALF;
+					effective |= MAX_HORIZONTAL;
+					effective &= ~(MAX_VERTICAL | MAX_BOTTOMHALF);
+				}
+			}
+		}
+
+		if (dest_head != -1)
+			/* tell wMaximizeWindow that we were using keyboard, not mouse,
+			 * so that it will use calculated head as destination for
+			 * move_half_max_between_heads feature, not from mouse pointer */
+			wMaximizeWindow(wwin, (effective | flags | MAX_KEYBOARD),
+					dest_head);
+		else if (!wPreferences.alt_half_maximize)
+			wUnmaximizeWindow(wwin);
+
+		return;
+	}
+
 	/* these alone mean vertical|horizontal toggle */
-	} else if ((effective == MAX_LEFTHALF) ||
+	if ((effective == MAX_LEFTHALF) ||
 			(effective == MAX_RIGHTHALF) ||
 			(effective == MAX_TOPHALF) ||
 			(effective == MAX_BOTTOMHALF))
 		wUnmaximizeWindow(wwin);
+
+	/* Following conditions might look complicated, but they are really simple:
+	 * allow fullscreen transition only for half maximized state (and not
+	 * corners) and only when requested state is also half maximized, but on
+	 * opposite side of the screen. As for corners, it is similar, but
+	 * expected is that only quarter maximized windows on corner can change
+	 * it's state to half maximized window, depending on direction. Note, that
+	 * MAX_KEYBOARD is passed to the wMaximizeWindow function, to preserve the
+	 * head, even if mouse was used for triggering the action. */
+
+	/* Quarters alternative transition. */
+	else if (wPreferences.alt_half_maximize &&
+			((requested & MAX_LEFTHALF && requested & MAX_TOPHALF &&
+			  current & MAX_RIGHTHALF && current & MAX_TOPHALF) ||
+			(requested & MAX_RIGHTHALF && requested & MAX_TOPHALF &&
+			  current & MAX_LEFTHALF && current & MAX_TOPHALF)))
+		wMaximizeWindow(wwin, (MAX_TOPHALF | MAX_HORIZONTAL | MAX_KEYBOARD),
+				head);
+	else if (wPreferences.alt_half_maximize &&
+			((requested & MAX_LEFTHALF && requested & MAX_BOTTOMHALF &&
+			 current & MAX_RIGHTHALF && current & MAX_BOTTOMHALF) ||
+			(requested & MAX_RIGHTHALF && requested & MAX_BOTTOMHALF &&
+			 current & MAX_LEFTHALF && current & MAX_BOTTOMHALF)))
+		wMaximizeWindow(wwin, (MAX_BOTTOMHALF | MAX_HORIZONTAL | MAX_KEYBOARD),
+				head);
+	else if (wPreferences.alt_half_maximize &&
+			((requested & MAX_LEFTHALF && requested & MAX_BOTTOMHALF &&
+			 current & MAX_LEFTHALF && current & MAX_TOPHALF) ||
+			(requested & MAX_LEFTHALF && requested & MAX_TOPHALF &&
+			 current & MAX_LEFTHALF && current & MAX_BOTTOMHALF)))
+		wMaximizeWindow(wwin, (MAX_LEFTHALF | MAX_VERTICAL| MAX_KEYBOARD),
+				head);
+	else if (wPreferences.alt_half_maximize &&
+			((requested & MAX_RIGHTHALF && requested & MAX_BOTTOMHALF &&
+			 current & MAX_RIGHTHALF && current & MAX_TOPHALF) ||
+			(requested & MAX_RIGHTHALF && requested & MAX_TOPHALF &&
+			 current & MAX_RIGHTHALF && current & MAX_BOTTOMHALF)))
+		wMaximizeWindow(wwin, (MAX_RIGHTHALF | MAX_VERTICAL| MAX_KEYBOARD),
+				head);
+
+	/* Half-maximized alternative transition */
+	else if (wPreferences.alt_half_maximize && (
+				(requested & MAX_LEFTHALF && requested & MAX_VERTICAL &&
+				 current & MAX_RIGHTHALF && current & MAX_VERTICAL) ||
+				(requested & MAX_RIGHTHALF && requested & MAX_VERTICAL &&
+				 current & MAX_LEFTHALF && current & MAX_VERTICAL) ||
+				(requested & MAX_TOPHALF && requested & MAX_HORIZONTAL &&
+				 current & MAX_BOTTOMHALF && current & MAX_HORIZONTAL) ||
+				(requested & MAX_BOTTOMHALF && requested & MAX_HORIZONTAL &&
+				 current & MAX_TOPHALF && current & MAX_HORIZONTAL)))
+		wMaximizeWindow(wwin, (MAX_HORIZONTAL|MAX_VERTICAL|MAX_KEYBOARD),
+				head);
+
 	else {
 		if ((requested == (MAX_HORIZONTAL | MAX_VERTICAL)) ||
 				(requested == MAX_MAXIMUS))
@@ -552,7 +665,7 @@ void handleMaximize(WWindow *wwin, int directions)
 				effective &= ~(MAX_TOPHALF | MAX_BOTTOMHALF);
 			effective &= ~MAX_MAXIMUS;
 		}
-		wMaximizeWindow(wwin, effective | flags);
+		wMaximizeWindow(wwin, effective | flags, head);
 	}
 }
 
@@ -726,8 +839,7 @@ void wFullscreenWindow(WWindow *wwin)
 
 	wWindowConfigureBorders(wwin);
 
-	ChangeStackingLevel(wwin->frame->core, WMNormalLevel);
-	wRaiseFrame(wwin->frame->core);
+	ChangeStackingLevel(wwin->frame->core, WMFullscreenLevel);
 
 	wwin->bfs_geometry.x = wwin->frame_x;
 	wwin->bfs_geometry.y = wwin->frame_y;
@@ -751,10 +863,15 @@ void wUnfullscreenWindow(WWindow *wwin)
 
 	wwin->flags.fullscreen = False;
 
-	if (WFLAGP(wwin, sunken))
-		ChangeStackingLevel(wwin->frame->core, WMSunkenLevel);
-	else if (WFLAGP(wwin, floating))
-		ChangeStackingLevel(wwin->frame->core, WMFloatingLevel);
+	if (wwin->frame->core->stacking->window_level == WMFullscreenLevel) {
+		if (WFLAGP(wwin, sunken)) {
+			ChangeStackingLevel(wwin->frame->core, WMSunkenLevel);
+		} else if (WFLAGP(wwin, floating)) {
+			ChangeStackingLevel(wwin->frame->core, WMFloatingLevel);
+		} else {
+			ChangeStackingLevel(wwin->frame->core, WMNormalLevel);
+		}
+	}
 
 	wWindowConfigure(wwin, wwin->bfs_geometry.x, wwin->bfs_geometry.y,
 			 wwin->bfs_geometry.width, wwin->bfs_geometry.height);
@@ -1135,7 +1252,7 @@ void wIconifyWindow(WWindow *wwin)
 		wwin->icon = icon_create_for_wwindow(wwin);
 		wwin->icon->mapped = 1;
 
-		/* extract the window screenshot everytime, as the option can be enable anytime */
+		/* extract the window screenshot every time, as the option can be enable anytime */
 		if (wwin->client_win && wwin->flags.mapped) {
 			RImage *mini_preview;
 			XImage *pimg;
@@ -1965,6 +2082,18 @@ void wMakeWindowVisible(WWindow *wwin)
 	}
 }
 
+void movePionterToWindowCenter(WWindow *wwin)
+{
+	if (!wPreferences.pointer_with_half_max_windows)
+		return;
+
+	XSelectInput(dpy, wwin->client_win, wwin->event_mask);
+	XWarpPointer(dpy, None, wwin->screen_ptr->root_win, 0, 0, 0, 0,
+			wwin->frame_x + wwin->client.width / 2,
+			wwin->frame_y + wwin->client.height / 2);
+	XFlush(dpy);
+}
+
 /*
  * Do the animation while shading (called with what = SHADE)
  * or unshading (what = UNSHADE).
