@@ -22,7 +22,7 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
-#ifdef SHAPE
+#ifdef USE_XSHAPE
 #include <X11/extensions/shape.h>
 #endif
 
@@ -44,23 +44,6 @@
 #include "wmspec.h"
 #include "misc.h"
 
-/****** Global Variables ******/
-
-/* contexts */
-extern XContext wWinContext;
-
-extern Atom _XA_WM_STATE;
-extern Atom _XA_WM_PROTOCOLS;
-extern Atom _XA_WM_COLORMAP_WINDOWS;
-
-extern Atom _XA_WINDOWMAKER_MENU;
-
-extern Atom _XA_GNUSTEP_WM_ATTR;
-extern Atom _XA_GNUSTEP_WM_RESIZEBAR;
-
-#ifdef SHAPE
-extern Bool wShapeSupported;
-#endif
 
 /*
  *--------------------------------------------------------------------
@@ -112,8 +95,9 @@ void wClientSetState(WWindow * wwin, int state, Window icon_win)
 	data[0] = (unsigned long)state;
 	data[1] = (unsigned long)icon_win;
 
-	XChangeProperty(dpy, wwin->client_win, _XA_WM_STATE, _XA_WM_STATE, 32,
-			PropModeReplace, (unsigned char *)data, 2);
+	XChangeProperty(dpy, wwin->client_win, w_global.atom.wm.state,
+			w_global.atom.wm.state, 32, PropModeReplace,
+			(unsigned char *)data, 2);
 }
 
 void wClientGetGravityOffsets(WWindow * wwin, int *ofs_x, int *ofs_y)
@@ -182,8 +166,8 @@ void wClientConfigure(WWindow * wwin, XConfigureRequestEvent * xcre)
 		XConfigureWindow(dpy, xcre->window, xcre->value_mask, &xwc);
 		return;
 	}
-#ifdef SHAPE
-	if (wShapeSupported) {
+#ifdef USE_XSHAPE
+	if (w_global.xext.shape.supported) {
 		int junk;
 		unsigned int ujunk;
 		int b_shaped;
@@ -199,7 +183,7 @@ void wClientConfigure(WWindow * wwin, XConfigureRequestEvent * xcre)
 		WWindow *sibling;
 
 		if ((xcre->value_mask & CWSibling) &&
-		    (XFindContext(dpy, xcre->above, wWinContext, (XPointer *) & desc) == XCSUCCESS)
+		    (XFindContext(dpy, xcre->above, w_global.context.client_win, (XPointer *) & desc) == XCSUCCESS)
 		    && (desc->parent_type == WCLASS_WINDOW)) {
 			sibling = desc->parent;
 			xwc.sibling = sibling->frame->core->window;
@@ -251,6 +235,10 @@ void wClientConfigure(WWindow * wwin, XConfigureRequestEvent * xcre)
 		else
 			nheight = wwin->frame->core->height - wwin->frame->top_width - wwin->frame->bottom_width;
 
+		/* Don't overwrite the saved geometry unnecessarily. */
+		if (!(xcre->value_mask & (CWX | CWY | CWWidth | CWHeight)))
+			return;
+
 		if (nwidth != wwin->old_geometry.width)
 			wwin->flags.maximized &= ~(MAX_HORIZONTAL | MAX_TOPHALF | MAX_BOTTOMHALF | MAX_MAXIMUS);
 		if (nheight != wwin->old_geometry.height)
@@ -269,7 +257,7 @@ void wClientSendProtocol(WWindow * wwin, Atom protocol, Time time)
 	XEvent event;
 
 	event.xclient.type = ClientMessage;
-	event.xclient.message_type = _XA_WM_PROTOCOLS;
+	event.xclient.message_type = w_global.atom.wm.protocols;
 	event.xclient.format = 32;
 	event.xclient.display = dpy;
 	event.xclient.window = wwin->client_win;
@@ -495,8 +483,8 @@ void wClientCheckProperty(WWindow * wwin, XPropertyEvent * event)
 			int foo;
 			unsigned bar;
 
-			XGetWindowAttributes(dpy, wwin->client_win, &attribs);
-			wClientGetNormalHints(wwin, &attribs, False, &foo, &foo, &bar, &bar);
+			if (XGetWindowAttributes(dpy, wwin->client_win, &attribs) != 0)
+				wClientGetNormalHints(wwin, &attribs, False, &foo, &foo, &bar, &bar);
 			/* TODO: should we check for consistency of the current
 			 * size against the new geometry hints? */
 		}
@@ -536,14 +524,14 @@ void wClientCheckProperty(WWindow * wwin, XPropertyEvent * event)
 				wwin->transient_for = new_owner;
 				if (new_owner == None) {
 					if (WFLAGP(wwin, no_miniaturizable)) {
-						WSETUFLAG(wwin, no_miniaturizable, 0);
-						WSETUFLAG(wwin, no_miniaturize_button, 0);
+						wwin->client_flags.no_miniaturizable = 0;
+						wwin->client_flags.no_miniaturize_button = 0;
 						if (wwin->frame)
 							wWindowConfigureBorders(wwin);
 					}
 				} else if (!WFLAGP(wwin, no_miniaturizable)) {
-					WSETUFLAG(wwin, no_miniaturizable, 1);
-					WSETUFLAG(wwin, no_miniaturize_button, 1);
+					wwin->client_flags.no_miniaturizable = 1;
+					wwin->client_flags.no_miniaturize_button = 1;
 					if (wwin->frame)
 						wWindowConfigureBorders(wwin);
 				}
@@ -552,21 +540,21 @@ void wClientCheckProperty(WWindow * wwin, XPropertyEvent * event)
 		break;
 
 	default:
-		if (event->atom == _XA_WM_PROTOCOLS) {
+		if (event->atom == w_global.atom.wm.protocols) {
 
 			PropGetProtocols(wwin->client_win, &wwin->protocols);
 
-			WSETUFLAG(wwin, kill_close, !wwin->protocols.DELETE_WINDOW);
+			wwin->client_flags.kill_close = !wwin->protocols.DELETE_WINDOW;
 
 			if (wwin->frame)
 				wWindowUpdateButtonImages(wwin);
 
-		} else if (event->atom == _XA_WM_COLORMAP_WINDOWS) {
+		} else if (event->atom == w_global.atom.wm.colormap_windows) {
 
 			GetColormapWindows(wwin);
 			wColormapInstallForWindow(wwin->screen_ptr, wwin);
 
-		} else if (event->atom == _XA_WINDOWMAKER_MENU) {
+		} else if (event->atom == w_global.atom.wmaker.menu) {
 			WApplication *wapp;
 
 			wapp = wApplicationOf(wwin->main_window);
@@ -584,8 +572,9 @@ void wClientCheckProperty(WWindow * wwin, XPropertyEvent * event)
 					wApplicationDestroy(wapp);
 					while (foo) {
 						if (foo->fake_group && foo->fake_group == fPtr) {
-							WSETUFLAG(foo, shared_appicon, 0);
+							foo->client_flags.shared_appicon = 0;
 							foo->fake_group = NULL;
+
 							if (foo->group_id != None)
 								foo->main_window = foo->group_id;
 							else if (foo->client_leader != None)
@@ -594,9 +583,9 @@ void wClientCheckProperty(WWindow * wwin, XPropertyEvent * event)
 								foo->main_window = foo->client_win;
 							else
 								foo->main_window = None;
-							if (foo->main_window) {
-								wapp = wApplicationCreate(foo);
-							}
+
+							if (foo->main_window)
+								wApplicationCreate(foo);
 						}
 						foo = foo->prev;
 					}
@@ -621,7 +610,7 @@ void wClientCheckProperty(WWindow * wwin, XPropertyEvent * event)
 				wSetFocusTo(wwin->screen_ptr, NULL);
 				wSetFocusTo(wwin->screen_ptr, wwin->screen_ptr->focused_window);
 			}
-		} else if (event->atom == _XA_GNUSTEP_WM_ATTR) {
+		} else if (event->atom == w_global.atom.gnustep.wm_attr) {
 			GNUstepWMAttributes *attr;
 
 			PropGetGNUstepWMAttr(wwin->client_win, &attr);
