@@ -25,6 +25,7 @@
 #include <X11/Xutil.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdio.h>
 
 #include "WindowMaker.h"
 #include "window.h"
@@ -33,13 +34,11 @@
 #include "dock.h"
 #include "dockedapp.h"
 #include "dialog.h"
-#include "funcs.h"
+#include "misc.h"
 #include "defaults.h"
 #include "framewin.h"
 #include "xinerama.h"
 
-/**** Global variables ****/
-extern WPreferences wPreferences;
 
 static void updateCommand(WAppIcon * icon, char *command)
 {
@@ -67,7 +66,7 @@ static void updatePasteCommand(WAppIcon * icon, char *command)
 	icon->paste_command = command;
 }
 
-#ifdef XDND
+#ifdef USE_DOCK_XDND
 static void updateDNDCommand(WAppIcon * icon, char *command)
 {
 	if (icon->dnd_command)
@@ -78,7 +77,7 @@ static void updateDNDCommand(WAppIcon * icon, char *command)
 	}
 	icon->dnd_command = command;
 }
-#endif				/* XDND */
+#endif	/* USE_DOCK_XDND */
 
 static void updateSettingsPanelIcon(AppSettingsPanel * panel)
 {
@@ -104,7 +103,7 @@ static void updateSettingsPanelIcon(AppSettingsPanel * panel)
 			color.green = 0xaa;
 			color.blue = 0xae;
 			color.alpha = 0;
-			pixmap = WMCreateBlendedPixmapFromFile(WMWidgetScreen(panel->win), path, &color);
+			pixmap = WMCreateScaledBlendedPixmapFromFile(WMWidgetScreen(panel->win), path, &color, 64, 64);
 			if (!pixmap) {
 				WMSetLabelImage(panel->iconLabel, NULL);
 			} else {
@@ -123,6 +122,9 @@ static void chooseIconCallback(WMWidget * self, void *clientData)
 	AppSettingsPanel *panel = (AppSettingsPanel *) clientData;
 	int result;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) self;
+
 	panel->choosingIcon = 1;
 
 	WMSetButtonEnabled(panel->browseBtn, False);
@@ -134,7 +136,6 @@ static void chooseIconCallback(WMWidget * self, void *clientData)
 	if (!panel->destroyed) {
 		if (result) {
 			WMSetTextFieldText(panel->iconField, file);
-			wfree(file);
 			updateSettingsPanelIcon(panel);
 		}
 
@@ -144,6 +145,8 @@ static void chooseIconCallback(WMWidget * self, void *clientData)
 		 * the icon chooser */
 		DestroyDockAppSettingsPanel(panel);
 	}
+	if (result)
+		wfree(file);
 }
 
 static void panelBtnCallback(WMWidget * self, void *data)
@@ -151,7 +154,6 @@ static void panelBtnCallback(WMWidget * self, void *data)
 	WMButton *btn = self;
 	AppSettingsPanel *panel = (AppSettingsPanel *) data;
 	char *text;
-	int done = 1;
 
 	if (panel->okBtn == btn) {
 		text = WMGetTextFieldText(panel->iconField);
@@ -168,8 +170,7 @@ static void panelBtnCallback(WMWidget * self, void *data)
 			snprintf(buf, len, _("Could not open specified icon file: %s"), text);
 			if (wMessageDialog(panel->wwin->screen_ptr, _("Error"), buf,
 					   _("OK"), _("Ignore"), NULL) == WAPRDefault) {
-				if (text)
-					wfree(text);
+				wfree(text);
 				wfree(buf);
 				return;
 			}
@@ -177,12 +178,15 @@ static void panelBtnCallback(WMWidget * self, void *data)
 		} else {
 			WAppIcon *aicon = panel->editedIcon;
 
+			// Cf dock.c:dockIconPaint(WAppIcon *aicon)?
 			if (aicon == aicon->icon->core->screen_ptr->clip_icon)
 				wClipIconPaint(aicon);
+			else if (wIsADrawer(aicon))
+				wDrawerIconPaint(aicon);
 			else
 				wAppIconPaint(aicon);
 
-			wDefaultChangeIcon(panel->wwin->screen_ptr, aicon->wm_instance, aicon->wm_class, text);
+			wDefaultChangeIcon(aicon->wm_instance, aicon->wm_class, text);
 		}
 		if (text)
 			wfree(text);
@@ -195,7 +199,7 @@ static void panelBtnCallback(WMWidget * self, void *data)
 			text = NULL;
 		}
 		updateCommand(panel->editedIcon, text);
-#ifdef XDND
+#ifdef USE_DOCK_XDND
 		/* cannot free text from this, because it will be not be duplicated
 		 * in updateDNDCommand */
 		text = WMGetTextFieldText(panel->dndCommandField);
@@ -209,8 +213,7 @@ static void panelBtnCallback(WMWidget * self, void *data)
 		panel->editedIcon->lock = WMGetButtonSelected(panel->lockBtn);
 	}
 
-	if (done)
-		DestroyDockAppSettingsPanel(panel);
+	DestroyDockAppSettingsPanel(panel);
 }
 
 #define PWIDTH	295
@@ -294,7 +297,7 @@ void ShowDockAppSettingsPanel(WAppIcon * aicon)
 	WMMapSubwidgets(panel->pasteCommandFrame);
 
 	panel->dndCommandFrame = WMCreateFrame(vbox);
-	WMSetFrameTitle(panel->dndCommandFrame, _("Command for files dropped with DND"));
+	WMSetFrameTitle(panel->dndCommandFrame, _("Command for dragged and dropped files"));
 	WMAddBoxSubview(vbox, WMWidgetView(panel->dndCommandFrame), False, True, 70, 70, 5);
 
 	panel->dndCommandField = WMCreateTextField(panel->dndCommandFrame);
@@ -304,12 +307,15 @@ void ShowDockAppSettingsPanel(WAppIcon * aicon)
 	panel->dndCommandLabel = WMCreateLabel(panel->dndCommandFrame);
 	WMResizeWidget(panel->dndCommandLabel, 256, 18);
 	WMMoveWidget(panel->dndCommandLabel, 10, 45);
-#ifdef XDND
+#ifdef USE_DOCK_XDND
 	WMSetTextFieldText(panel->dndCommandField, aicon->dnd_command);
 	WMSetLabelText(panel->dndCommandLabel, _("%d will be replaced with the file name"));
 #else
 	WMSetTextFieldEditable(panel->dndCommandField, False);
-	WMSetLabelText(panel->dndCommandLabel, _("DND support was not compiled in"));
+	WMSetLabelText(panel->dndCommandLabel, _("XDnD support was not compiled in"));
+
+	WMSetFrameTitleColor(panel->dndCommandFrame, WMDarkGrayColor(scr->wmscreen));
+	WMSetLabelTextColor(panel->dndCommandLabel, WMDarkGrayColor(scr->wmscreen));
 #endif
 	WMMapSubwidgets(panel->dndCommandFrame);
 
