@@ -43,8 +43,6 @@
 #include "misc.h"
 #include "event.h"
 
-#define DBLCLICK_TIME wPreferences.dblclick_time
-
 
 static void handleExpose(WObjDescriptor * desc, XEvent * event);
 static void handleButtonExpose(WObjDescriptor * desc, XEvent * event);
@@ -415,6 +413,7 @@ void wFrameWindowUpdateBorders(WFrameWindow * fwin, int flags)
 	checkTitleSize(fwin);
 
 	allocFrameBorderPixel(fwin->colormap, WMGetColorRGBDescription(scr->frame_border_color), &fwin->border_pixel);
+	allocFrameBorderPixel(fwin->colormap, WMGetColorRGBDescription(scr->frame_focused_border_color), &fwin->focused_border_pixel);
 	allocFrameBorderPixel(fwin->colormap, WMGetColorRGBDescription(scr->frame_selected_border_color), &fwin->selected_border_pixel);
 
 	if (flags & WFF_SELECTED) {
@@ -422,8 +421,13 @@ void wFrameWindowUpdateBorders(WFrameWindow * fwin, int flags)
 			XSetWindowBorder(dpy, fwin->core->window, *fwin->selected_border_pixel);
 	}
 	else {
-		if (fwin->border_pixel)
-			XSetWindowBorder(dpy, fwin->core->window, *fwin->border_pixel);
+		if (fwin->flags.state == WS_FOCUSED) {
+			if (fwin->focused_border_pixel)
+				XSetWindowBorder(dpy, fwin->core->window, *fwin->focused_border_pixel);
+		} else {
+			if (fwin->border_pixel)
+				XSetWindowBorder(dpy, fwin->core->window, *fwin->border_pixel);
+		}
 	}
 }
 
@@ -477,6 +481,13 @@ void wFrameWindowChangeState(WFrameWindow * fwin, int state)
 	fwin->flags.state = state;
 	fwin->flags.need_texture_change = 1;
 
+	if (fwin->flags.state == WS_FOCUSED) {
+		if (fwin->focused_border_pixel)
+			XSetWindowBorder(dpy, fwin->core->window, *fwin->focused_border_pixel);
+	} else {
+		if (fwin->border_pixel)
+			XSetWindowBorder(dpy, fwin->core->window, *fwin->border_pixel);
+	}
 	wFrameWindowPaint(fwin);
 }
 
@@ -1192,12 +1203,12 @@ void wFrameWindowResize(WFrameWindow * fwin, int width, int height)
 
 int wFrameWindowChangeTitle(WFrameWindow *fwin, const char *new_title)
 {
+	if (new_title == NULL)
+		return 0;
+
 	/* check if the title is the same as before */
 	if (fwin->title) {
-		if (new_title && (strcmp(fwin->title, new_title) == 0))
-			return 0;
-	} else {
-		if (!new_title)
+		if (strcmp(fwin->title, new_title) == 0)
 			return 0;
 	}
 
@@ -1378,6 +1389,9 @@ static void handleButtonExpose(WObjDescriptor * desc, XEvent * event)
 	WFrameWindow *fwin = (WFrameWindow *) desc->parent;
 	WCoreWindow *button = (WCoreWindow *) desc->self;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) event;
+
 #ifdef XKB_BUTTON_HINT
 	if (button == fwin->language_button) {
 		if (wPreferences.modelock)
