@@ -63,6 +63,8 @@ enum {
 	WKBD_RBCMAXIMIZE,
 	WKBD_MAXIMUS,
 	WKBD_SELECT,
+	WKBD_KEEP_ON_TOP,
+	WKBD_KEEP_AT_BOTTOM,
 	WKBD_OMNIPRESENT,
 	WKBD_RAISE,
 	WKBD_LOWER,
