@@ -32,7 +32,6 @@
 
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
-#include <X11/Xarch.h>
 #include <string.h>
 
 #include <WINGs/WUtil.h>
@@ -50,9 +49,6 @@
 #include "xinerama.h"
 #include "properties.h"
 
-/* Global variables */
-extern Atom _XA_WM_DELETE_WINDOW;
-extern Time LastTimestamp;
 
 /* Root Window Properties */
 static Atom net_supported;
@@ -64,6 +60,7 @@ static Atom net_desktop_viewport;
 static Atom net_current_desktop;
 static Atom net_desktop_names;
 static Atom net_active_window;
+static Atom net_workarea;
 static Atom net_supporting_wm_check;
 static Atom net_virtual_roots;	/* N/A */
 static Atom net_desktop_layout;	/* XXX */
@@ -88,6 +85,12 @@ static Atom net_wm_window_type_menu;
 static Atom net_wm_window_type_utility;
 static Atom net_wm_window_type_splash;
 static Atom net_wm_window_type_dialog;
+static Atom net_wm_window_type_dropdown_menu;
+static Atom net_wm_window_type_popup_menu;
+static Atom net_wm_window_type_tooltip;
+static Atom net_wm_window_type_notification;
+static Atom net_wm_window_type_combo;
+static Atom net_wm_window_type_dnd;
 static Atom net_wm_window_type_normal;
 static Atom net_wm_state;
 static Atom net_wm_state_modal;	/* XXX: what is this?!? */
@@ -112,12 +115,13 @@ static Atom net_wm_action_maximize_vert;
 static Atom net_wm_action_fullscreen;
 static Atom net_wm_action_change_desktop;
 static Atom net_wm_action_close;
-static Atom net_wm_strut;	/* XXX: see net_workarea */
+static Atom net_wm_strut;
 static Atom net_wm_strut_partial;	/* TODO: doesn't really fit into the current strut scheme */
 static Atom net_wm_icon_geometry;	/* FIXME: should work together with net_wm_handled_icons, gnome-panel-2.2.0.1 doesn't use _NET_WM_HANDLED_ICONS, thus present situation. */
 static Atom net_wm_icon;
 static Atom net_wm_pid;		/* TODO */
 static Atom net_wm_handled_icons;	/* FIXME: see net_wm_icon_geometry */
+static Atom net_wm_window_opacity;
 
 static Atom net_frame_extents;
 
@@ -141,6 +145,7 @@ static atomitem_t atomNames[] = {
 	{"_NET_CURRENT_DESKTOP", &net_current_desktop},
 	{"_NET_DESKTOP_NAMES", &net_desktop_names},
 	{"_NET_ACTIVE_WINDOW", &net_active_window},
+	{"_NET_WORKAREA", &net_workarea},
 	{"_NET_SUPPORTING_WM_CHECK", &net_supporting_wm_check},
 	{"_NET_VIRTUAL_ROOTS", &net_virtual_roots},
 	{"_NET_DESKTOP_LAYOUT", &net_desktop_layout},
@@ -163,6 +168,12 @@ static atomitem_t atomNames[] = {
 	{"_NET_WM_WINDOW_TYPE_UTILITY", &net_wm_window_type_utility},
 	{"_NET_WM_WINDOW_TYPE_SPLASH", &net_wm_window_type_splash},
 	{"_NET_WM_WINDOW_TYPE_DIALOG", &net_wm_window_type_dialog},
+	{"_NET_WM_WINDOW_TYPE_DROPDOWN_MENU", &net_wm_window_type_dropdown_menu},
+	{"_NET_WM_WINDOW_TYPE_POPUP_MENU", &net_wm_window_type_popup_menu},
+	{"_NET_WM_WINDOW_TYPE_TOOLTIP", &net_wm_window_type_tooltip},
+	{"_NET_WM_WINDOW_TYPE_NOTIFICATION", &net_wm_window_type_notification},
+	{"_NET_WM_WINDOW_TYPE_COMBO", &net_wm_window_type_combo},
+	{"_NET_WM_WINDOW_TYPE_DND", &net_wm_window_type_dnd},
 	{"_NET_WM_WINDOW_TYPE_NORMAL", &net_wm_window_type_normal},
 	{"_NET_WM_STATE", &net_wm_state},
 	{"_NET_WM_STATE_MODAL", &net_wm_state_modal},
@@ -193,6 +204,7 @@ static atomitem_t atomNames[] = {
 	{"_NET_WM_ICON", &net_wm_icon},
 	{"_NET_WM_PID", &net_wm_pid},
 	{"_NET_WM_HANDLED_ICONS", &net_wm_handled_icons},
+	{"_NET_WM_WINDOW_OPACITY", &net_wm_window_opacity},
 
 	{"_NET_FRAME_EXTENTS", &net_frame_extents},
 
@@ -201,12 +213,25 @@ static atomitem_t atomNames[] = {
 	{"UTF8_STRING", &utf8_string},
 };
 
-#define atomNr (sizeof(atomNames)/sizeof(atomNames[0]))
-
-#define _NET_WM_STATE_REMOVE 0
 #define _NET_WM_STATE_ADD 1
 #define _NET_WM_STATE_TOGGLE 2
 
+#if 0
+/*
+ * These constant provide information on the kind of window move/resize when
+ * it is initiated by the application instead of by WindowMaker. They are
+ * parameter for the client message _NET_WM_MOVERESIZE, as defined by the
+ * FreeDesktop wm-spec standard:
+ *   http://standards.freedesktop.org/wm-spec/1.5/ar01s04.html
+ *
+ * Today, WindowMaker does not support this at all (the corresponding Atom
+ * is not added to the list in setSupportedHints), probably because there is
+ * nothing it needs to do about it, the application is assumed to know what
+ * it is doing, and WindowMaker won't get in the way.
+ *
+ * The definition of the constants (taken from the standard) are disabled to
+ * avoid a spurious warning (-Wunused-macros).
+ */
 #define _NET_WM_MOVERESIZE_SIZE_TOPLEFT      0
 #define _NET_WM_MOVERESIZE_SIZE_TOP          1
 #define _NET_WM_MOVERESIZE_SIZE_TOPRIGHT     2
@@ -218,6 +243,7 @@ static atomitem_t atomNames[] = {
 #define _NET_WM_MOVERESIZE_MOVE              8	/* movement only */
 #define _NET_WM_MOVERESIZE_SIZE_KEYBOARD     9	/* size via keyboard */
 #define _NET_WM_MOVERESIZE_MOVE_KEYBOARD    10	/* move via keyboard */
+#endif
 
 static void observer(void *self, WMNotification *notif);
 static void wsobserver(void *self, WMNotification *notif);
@@ -238,7 +264,7 @@ typedef struct NetData {
 
 static void setSupportedHints(WScreen *scr)
 {
-	Atom atom[atomNr];
+	Atom atom[wlengthof(atomNames)];
 	int i = 0;
 
 	/* set supported hints list */
@@ -252,6 +278,7 @@ static void setSupportedHints(WScreen *scr)
 	atom[i++] = net_current_desktop;
 	atom[i++] = net_desktop_names;
 	atom[i++] = net_active_window;
+	atom[i++] = net_workarea;
 	atom[i++] = net_supporting_wm_check;
 	atom[i++] = net_showing_desktop;
 #if 0
@@ -266,6 +293,12 @@ static void setSupportedHints(WScreen *scr)
 	atom[i++] = net_wm_window_type_utility;
 	atom[i++] = net_wm_window_type_splash;
 	atom[i++] = net_wm_window_type_dialog;
+	atom[i++] = net_wm_window_type_dropdown_menu;
+	atom[i++] = net_wm_window_type_popup_menu;
+	atom[i++] = net_wm_window_type_tooltip;
+	atom[i++] = net_wm_window_type_notification;
+	atom[i++] = net_wm_window_type_combo;
+	atom[i++] = net_wm_window_type_dnd;
 	atom[i++] = net_wm_window_type_normal;
 
 	atom[i++] = net_wm_state;
@@ -297,6 +330,7 @@ static void setSupportedHints(WScreen *scr)
 	atom[i++] = net_wm_icon_geometry;
 	atom[i++] = net_wm_icon;
 	atom[i++] = net_wm_handled_icons;
+	atom[i++] = net_wm_window_opacity;
 
 	atom[i++] = net_frame_extents;
 
@@ -323,7 +357,6 @@ void wNETWMUpdateDesktop(WScreen *scr)
 
 	count = scr->workspace_count * 2;
 	views = wmalloc(sizeof(long) * count);
-	/*memset(views, 0, sizeof(long) * count); */
 	sizes[0] = scr->scr_width;
 	sizes[1] = scr->scr_height;
 
@@ -477,6 +510,34 @@ static void updateIconImage(WWindow *wwin)
 	}
 }
 
+static void updateWindowOpacity(WWindow *wwin)
+{
+	Atom type;
+	int format;
+	unsigned long items, rest;
+	unsigned long *property;
+
+	if (!wwin->frame)
+		return;
+
+	/* We don't care about this ourselves, but other programs need us to copy
+	 * this to the frame window. */
+	if (XGetWindowProperty(dpy, wwin->client_win, net_wm_window_opacity, 0L, 1L,
+				False, XA_CARDINAL, &type, &format, &items, &rest,
+				(unsigned char **)&property) != Success)
+		return;
+
+	if (type == None) {
+		XDeleteProperty(dpy, wwin->frame->core->window, net_wm_window_opacity);
+	} else if (type == XA_CARDINAL && format == 32 && items == 1 && property) {
+		XChangeProperty(dpy, wwin->frame->core->window, net_wm_window_opacity,
+				XA_CARDINAL, 32, PropModeReplace, (unsigned char *)property, 1L);
+	}
+
+	if (property)
+		XFree(property);
+}
+
 static void updateShowDesktop(WScreen *scr, Bool show)
 {
 	long foo;
@@ -537,19 +598,19 @@ void wNETWMInitStuff(WScreen *scr)
 
 #ifdef HAVE_XINTERNATOMS
 	{
-		Atom atoms[atomNr];
-		char *names[atomNr];
+		Atom atoms[wlengthof(atomNames)];
+		char *names[wlengthof(atomNames)];
 
-		for (i = 0; i < atomNr; ++i)
+		for (i = 0; i < wlengthof(atomNames); ++i)
 			names[i] = atomNames[i].name;
 
-		XInternAtoms(dpy, &names[0], atomNr, False, atoms);
-		for (i = 0; i < atomNr; ++i)
+		XInternAtoms(dpy, &names[0], wlengthof(atomNames), False, atoms);
+		for (i = 0; i < wlengthof(atomNames); ++i)
 			*atomNames[i].atom = atoms[i];
 
 	}
 #else
-	for (i = 0; i < atomNr; i++)
+	for (i = 0; i < wlengthof(atomNames); i++)
 		*atomNames[i].atom = XInternAtom(dpy, atomNames[i].name, False);
 #endif
 
@@ -588,7 +649,7 @@ void wNETWMCleanup(WScreen *scr)
 {
 	int i;
 
-	for (i = 0; i < atomNr; i++)
+	for (i = 0; i < wlengthof(atomNames); i++)
 		XDeleteProperty(dpy, scr->root_win, *atomNames[i].atom);
 }
 
@@ -638,6 +699,71 @@ void wNETWMUpdateActions(WWindow *wwin, Bool del)
 			XA_ATOM, 32, PropModeReplace, (unsigned char *)action, i);
 }
 
+void wNETWMUpdateWorkarea(WScreen *scr)
+{
+	WArea total_usable;
+	int nb_workspace;
+
+	if (!scr->netdata) {
+		/* If the _NET_xxx were not initialised, it not necessary to do anything */
+		return;
+	}
+
+	if (!scr->usableArea) {
+		/* If we don't have any info, we fall back on using the complete screen area */
+		total_usable.x1 = 0;
+		total_usable.y1 = 0;
+		total_usable.x2 = scr->scr_width;
+		total_usable.y2 = scr->scr_height;
+
+	} else {
+		int i;
+
+		/*
+		 * the _NET_WORKAREA is supposed to contain the total area of the screen that
+		 * is usable, so we merge the areas from all xrandr sub-screens
+		 */
+		total_usable = scr->usableArea[0];
+
+		for (i = 1; i < wXineramaHeads(scr); i++) {
+			/* The merge is not subtle because _NET_WORKAREA does not need more */
+			if (scr->usableArea[i].x1 < total_usable.x1)
+				total_usable.x1 = scr->usableArea[i].x1;
+
+			if (scr->usableArea[i].y1 < total_usable.y1)
+				total_usable.y1 = scr->usableArea[i].y1;
+
+			if (scr->usableArea[i].x2 > total_usable.x2)
+				total_usable.x2 = scr->usableArea[i].x2;
+
+			if (scr->usableArea[i].y2 > total_usable.y2)
+				total_usable.y2 = scr->usableArea[i].y2;
+		}
+
+	}
+
+	/* We are expected to repeat the information for each workspace */
+	if (scr->workspace_count == 0)
+		nb_workspace = 1;
+	else
+		nb_workspace = scr->workspace_count;
+
+	{
+		long property_value[nb_workspace * 4];
+		int i;
+
+		for (i = 0; i < nb_workspace; i++) {
+			property_value[4 * i + 0] = total_usable.x1;
+			property_value[4 * i + 1] = total_usable.y1;
+			property_value[4 * i + 2] = total_usable.x2 - total_usable.x1;
+			property_value[4 * i + 3] = total_usable.y2 - total_usable.y1;
+		}
+
+		XChangeProperty(dpy, scr->root_win, net_workarea, XA_CARDINAL, 32, PropModeReplace,
+		                (unsigned char *) property_value, nb_workspace * 4);
+	}
+}
+
 Bool wNETWMGetUsableArea(WScreen *scr, int head, WArea *area)
 {
 	WReservedArea *cur;
@@ -773,7 +899,7 @@ static void updateWorkspaceNames(WScreen *scr)
 			PropModeReplace, (unsigned char *)buf, len);
 }
 
-static void updateFocusHint(WScreen *scr, WWindow *wwin)
+static void updateFocusHint(WScreen *scr)
 {				/* changeable */
 	Window window;
 
@@ -933,6 +1059,16 @@ static int getWindowLayer(WWindow *wwin)
 				layer = WMFullscreenLevel;
 		}
 		/* //layer = WMPopUpLevel; // this seems a bad idea -Dan */
+	} else if (wwin->type == net_wm_window_type_dropdown_menu) {
+		layer = WMSubmenuLevel;
+	} else if (wwin->type == net_wm_window_type_popup_menu) {
+		layer = WMSubmenuLevel;
+	} else if (wwin->type == net_wm_window_type_tooltip) {
+	} else if (wwin->type == net_wm_window_type_notification) {
+		layer = WMPopUpLevel;
+	} else if (wwin->type == net_wm_window_type_combo) {
+		layer = WMSubmenuLevel;
+	} else if (wwin->type == net_wm_window_type_dnd) {
 	} else if (wwin->type == net_wm_window_type_normal) {
 	}
 
@@ -983,9 +1119,11 @@ static void doStateAtom(WWindow *wwin, Atom state, int set, Bool init)
 			wwin->flags.maximized |= (set ? MAX_VERTICAL : 0);
 		} else {
 			if (set)
-				wMaximizeWindow(wwin, wwin->flags.maximized | MAX_VERTICAL);
+				wMaximizeWindow(wwin, wwin->flags.maximized | MAX_VERTICAL,
+						wGetHeadForWindow(wwin));
 			else
-				wMaximizeWindow(wwin, wwin->flags.maximized & ~MAX_VERTICAL);
+				wMaximizeWindow(wwin, wwin->flags.maximized & ~MAX_VERTICAL,
+						wGetHeadForWindow(wwin));
 		}
 	} else if (state == net_wm_state_maximized_horz) {
 		if (set == _NET_WM_STATE_TOGGLE)
@@ -995,9 +1133,11 @@ static void doStateAtom(WWindow *wwin, Atom state, int set, Bool init)
 			wwin->flags.maximized |= (set ? MAX_HORIZONTAL : 0);
 		} else {
 			if (set)
-				wMaximizeWindow(wwin, wwin->flags.maximized | MAX_HORIZONTAL);
+				wMaximizeWindow(wwin, wwin->flags.maximized | MAX_HORIZONTAL,
+						wGetHeadForWindow(wwin));
 			else
-				wMaximizeWindow(wwin, wwin->flags.maximized & ~MAX_HORIZONTAL);
+				wMaximizeWindow(wwin, wwin->flags.maximized & ~MAX_HORIZONTAL,
+						wGetHeadForWindow(wwin));
 		}
 	} else if (state == net_wm_state_hidden) {
 		if (set == _NET_WM_STATE_TOGGLE)
@@ -1047,7 +1187,7 @@ static void doStateAtom(WWindow *wwin, Atom state, int set, Bool init)
 
 	} else {
 #ifdef DEBUG_WMSPEC
-		wmessage("doStateAtom unknown atom %s set %d\n", XGetAtomName(dpy, state), set);
+		wmessage("doStateAtom unknown atom %s set %d", XGetAtomName(dpy, state), set);
 #endif
 	}
 }
@@ -1097,8 +1237,8 @@ static Bool handleWindowType(WWindow *wwin, Atom type, int *layer)
 		wwin->client_flags.skip_switchpanel = 1;
 		wwin->client_flags.dont_move_off = 1;
 		wwin->flags.net_skip_pager = 1;
-	} else if (type == net_wm_window_type_toolbar) {
-		wwin->client_flags.no_titlebar = 1;
+	} else if (type == net_wm_window_type_toolbar ||
+	           type == net_wm_window_type_menu) {
 		wwin->client_flags.no_resizable = 1;
 		wwin->client_flags.no_miniaturizable = 1;
 		wwin->client_flags.no_resizebar = 1;
@@ -1107,7 +1247,9 @@ static Bool handleWindowType(WWindow *wwin, Atom type, int *layer)
 		wwin->client_flags.skip_switchpanel = 1;
 		wwin->client_flags.dont_move_off = 1;
 		wwin->client_flags.no_appicon = 1;
-	} else if (type == net_wm_window_type_menu) {
+	} else if (type == net_wm_window_type_dropdown_menu ||
+			type == net_wm_window_type_popup_menu ||
+			type == net_wm_window_type_combo) {
 		wwin->client_flags.no_titlebar = 1;
 		wwin->client_flags.no_resizable = 1;
 		wwin->client_flags.no_miniaturizable = 1;
@@ -1136,6 +1278,48 @@ static Bool handleWindowType(WWindow *wwin, Atom type, int *layer)
 		   // wwin->client_flags.skip_window_list = 1;
 		   // wwin->client_flags.no_appicon = 1;
 		 */
+	} else if (wwin->type == net_wm_window_type_tooltip) {
+		wwin->client_flags.no_titlebar = 1;
+		wwin->client_flags.no_resizable = 1;
+		wwin->client_flags.no_miniaturizable = 1;
+		wwin->client_flags.no_resizebar = 1;
+		wwin->client_flags.no_shadeable = 1;
+		wwin->client_flags.no_movable = 1;
+		wwin->client_flags.skip_window_list = 1;
+		wwin->client_flags.skip_switchpanel = 1;
+		wwin->client_flags.dont_move_off = 1;
+		wwin->client_flags.no_appicon = 1;
+		wwin->client_flags.no_focusable = 1;
+		wwin->flags.net_skip_pager = 1;
+	} else if (wwin->type == net_wm_window_type_notification) {
+		wwin->client_flags.no_titlebar = 1;
+		wwin->client_flags.no_resizable = 1;
+		wwin->client_flags.no_miniaturizable = 1;
+		wwin->client_flags.no_border = 1;
+		wwin->client_flags.no_resizebar = 1;
+		wwin->client_flags.no_shadeable = 1;
+		wwin->client_flags.no_movable = 1;
+		wwin->client_flags.omnipresent = 1;
+		wwin->client_flags.skip_window_list = 1;
+		wwin->client_flags.skip_switchpanel = 1;
+		wwin->client_flags.dont_move_off = 1;
+		wwin->client_flags.no_hide_others= 1;
+		wwin->client_flags.no_appicon = 1;
+		wwin->client_flags.no_focusable = 1;
+		wwin->flags.net_skip_pager = 1;
+	} else if (wwin->type == net_wm_window_type_dnd) {
+		wwin->client_flags.no_titlebar = 1;
+		wwin->client_flags.no_resizable = 1;
+		wwin->client_flags.no_miniaturizable = 1;
+		wwin->client_flags.no_border = 1;
+		wwin->client_flags.no_resizebar = 1;
+		wwin->client_flags.no_shadeable = 1;
+		wwin->client_flags.no_movable = 1;
+		wwin->client_flags.skip_window_list = 1;
+		wwin->client_flags.skip_switchpanel = 1;
+		wwin->client_flags.dont_move_off = 1;
+		wwin->client_flags.no_appicon = 1;
+		wwin->flags.net_skip_pager = 1;
 	} else if (type == net_wm_window_type_normal) {
 	} else {
 		ret = False;
@@ -1301,11 +1485,21 @@ void wNETWMCheckInitialClientState(WWindow *wwin)
 
 	wNETWMShowingDesktop(wwin->screen_ptr, False);
 
+	updateWindowType(wwin);
 	updateNetIconInfo(wwin);
 	updateIconImage(wwin);
 }
 
-static void handleDesktopNames(XClientMessageEvent *event, WScreen *scr)
+void wNETWMCheckInitialFrameState(WWindow *wwin)
+{
+#ifdef DEBUG_WMSPEC
+	wmessage("wNETWMCheckInitialFrameState");
+#endif
+
+	updateWindowOpacity(wwin);
+}
+
+static void handleDesktopNames(WScreen *scr)
 {
 	unsigned long nitems_ret, bytes_after_ret;
 	char *data, *names[32];
@@ -1340,10 +1534,9 @@ Bool wNETWMProcessClientMessage(XClientMessageEvent *event)
 {
 	WScreen *scr;
 	WWindow *wwin;
-	Bool done = True;
 
 #ifdef DEBUG_WMSPEC
-	wmessage("processClientMessage type %s\n", XGetAtomName(dpy, event->message_type));
+	wmessage("processClientMessage type %s", XGetAtomName(dpy, event->message_type));
 #endif
 
 	scr = wScreenForWindow(event->window);
@@ -1351,6 +1544,8 @@ Bool wNETWMProcessClientMessage(XClientMessageEvent *event)
 		/* generic client messages */
 		if (event->message_type == net_current_desktop) {
 			wWorkspaceChange(scr, event->data.l[0]);
+			return True;
+
 		} else if (event->message_type == net_number_of_desktops) {
 			long value;
 
@@ -1371,16 +1566,16 @@ Bool wNETWMProcessClientMessage(XClientMessageEvent *event)
 				if (rebuild)
 					updateWorkspaceCount(scr);
 			}
+			return True;
+
 		} else if (event->message_type == net_showing_desktop) {
 			wNETWMShowingDesktop(scr, event->data.l[0]);
-		} else if (event->message_type == net_desktop_names) {
-			handleDesktopNames(event, scr);
-		} else {
-			done = False;
-		}
-
-		if (done)
 			return True;
+
+		} else if (event->message_type == net_desktop_names) {
+			handleDesktopNames(scr);
+			return True;
+		}
 	}
 
 	/* window specific client messages */
@@ -1403,17 +1598,22 @@ Bool wNETWMProcessClientMessage(XClientMessageEvent *event)
 				wNETWMShowingDesktop(scr, False);
 				wMakeWindowVisible(wwin);
 		}
+		return True;
+
 	} else if (event->message_type == net_close_window) {
 		if (!WFLAGP(wwin, no_closable)) {
 			if (wwin->protocols.DELETE_WINDOW)
-				wClientSendProtocol(wwin, _XA_WM_DELETE_WINDOW, LastTimestamp);
+				wClientSendProtocol(wwin, w_global.atom.wm.delete_window,
+										  w_global.timestamp.last_event);
 		}
+		return True;
+
 	} else if (event->message_type == net_wm_state) {
 		int maximized = wwin->flags.maximized;
 		long set = event->data.l[0];
 
 #ifdef DEBUG_WMSPEC
-		wmessage("net_wm_state set %ld a1 %s a2 %s\n", set,
+		wmessage("net_wm_state set %ld a1 %s a2 %s", set,
 		       XGetAtomName(dpy, event->data.l[1]), XGetAtomName(dpy, event->data.l[2]));
 #endif
 
@@ -1426,14 +1626,20 @@ Bool wNETWMProcessClientMessage(XClientMessageEvent *event)
 				wwin->flags.maximized = maximized;
 				wUnmaximizeWindow(wwin);
 			} else {
-				wMaximizeWindow(wwin, wwin->flags.maximized);
+				wMaximizeWindow(wwin, wwin->flags.maximized, 
+						wGetHeadForWindow(wwin));
 			}
 		}
 		updateStateHint(wwin, False, False);
+		return True;
+
 	} else if (event->message_type == net_wm_handled_icons || event->message_type == net_wm_icon_geometry) {
 		updateNetIconInfo(wwin);
+		return True;
+
 	} else if (event->message_type == net_wm_desktop) {
 		long desktop = event->data.l[0];
+
 		if (desktop == -1) {
 			wWindowSetOmnipresent(wwin, True);
 		} else {
@@ -1441,17 +1647,16 @@ Bool wNETWMProcessClientMessage(XClientMessageEvent *event)
 				wWindowSetOmnipresent(wwin, False);
 			wWindowChangeWorkspace(wwin, desktop);
 		}
-	} else {
-		done = False;
+		return True;
 	}
 
-	return done;
+	return False;
 }
 
 void wNETWMCheckClientHintChange(WWindow *wwin, XPropertyEvent *event)
 {
 #ifdef DEBUG_WMSPEC
-	wmessage("clientHintChange type %s\n", XGetAtomName(dpy, event->atom));
+	wmessage("clientHintChange type %s", XGetAtomName(dpy, event->atom));
 #endif
 
 	if (event->atom == net_wm_strut || event->atom == net_wm_strut_partial) {
@@ -1474,6 +1679,8 @@ void wNETWMCheckClientHintChange(WWindow *wwin, XPropertyEvent *event)
 		}
 	} else if (event->atom == net_wm_icon) {
 		updateIconImage(wwin);
+	} else if (event->atom == net_wm_window_opacity) {
+		updateWindowOpacity(wwin);
 	}
 }
 
@@ -1562,7 +1769,7 @@ static void observer(void *self, WMNotification *notif)
 		updateClientListStacking(wwin->screen_ptr, NULL);
 		updateStateHint(wwin, False, False);
 	} else if (strcmp(name, WMNChangedFocus) == 0) {
-		updateFocusHint(ndata->scr, wwin);
+		updateFocusHint(ndata->scr);
 	} else if (strcmp(name, WMNChangedWorkspace) == 0 && wwin) {
 		updateWorkspaceHint(wwin, False, False);
 		updateStateHint(wwin, True, False);
@@ -1576,12 +1783,17 @@ static void wsobserver(void *self, WMNotification *notif)
 	WScreen *scr = (WScreen *) WMGetNotificationObject(notif);
 	const char *name = WMGetNotificationName(notif);
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) self;
+
 	if (strcmp(name, WMNWorkspaceCreated) == 0) {
 		updateWorkspaceCount(scr);
 		updateWorkspaceNames(scr);
+		wNETWMUpdateWorkarea(scr);
 	} else if (strcmp(name, WMNWorkspaceDestroyed) == 0) {
 		updateWorkspaceCount(scr);
 		updateWorkspaceNames(scr);
+		wNETWMUpdateWorkarea(scr);
 	} else if (strcmp(name, WMNWorkspaceChanged) == 0) {
 		updateCurrentWorkspace(scr);
 	} else if (strcmp(name, WMNWorkspaceNameChanged) == 0) {
