@@ -50,6 +50,9 @@ extern WMTreeNode *menu;
 
 extern char *env_lang, *env_ctry, *env_enc, *env_mod;
 
+/* Type for the call-back function to add a menu entry to the current menu */
+typedef void cb_add_menu_entry(WMMenuEntry *entry);
+
 /* wmmenu_misc.c
  */
 void  parse_locale(const char *what, char **env_lang, char **env_ctry, char **env_enc, char **env_mod);
@@ -58,8 +61,8 @@ Bool fileInPath(const char *file);
 
 /* implemented parsers
  */
-void parse_xdg(const char *file, void (*addWMMenuEntryCallback)(WMMenuEntry *aEntry));
-void parse_wmconfig(const char *file, void (*addWMMenuEntryCallback)(WMMenuEntry *aEntry));
+void parse_xdg(const char *file, cb_add_menu_entry *addWMMenuEntryCallback);
+void parse_wmconfig(const char *file, cb_add_menu_entry *addWMMenuEntryCallback);
 Bool wmconfig_validate_file(const char *filename, const struct stat *st, int tflags, struct FTW *ftw);
 
 #endif  /* WMMENUGEN_H */
