@@ -24,6 +24,8 @@
 /*
  * TODO: rewrite, too dirty
  */
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -37,9 +39,7 @@
 #include <sys/types.h>
 #include <ctype.h>
 
-#include "../config.h"
-
-#ifdef XINERAMA
+#ifdef USE_XINERAMA
 # ifdef SOLARIS_XINERAMA	/* sucks */
 #  include <X11/extensions/xinerama.h>
 # else
@@ -47,11 +47,12 @@
 # endif
 #endif
 
+#ifdef HAVE_STDNORETURN
+#include <stdnoreturn.h>
+#endif
+
 #include "../src/wconfig.h"
 
-#ifndef GLOBAL_DEFAULTS_SUBDIR
-#define GLOBAL_DEFAULTS_SUBDIR "WindowMaker"
-#endif
 
 #include <WINGs/WINGs.h>
 #include <wraster.h>
@@ -74,14 +75,14 @@ int scrX, scrY;
 WXineramaInfo xineInfo;
 
 Bool smooth = False;
-#ifdef XINERAMA
+#ifdef USE_XINERAMA
 Bool xineStretch = False;
 #endif
 
 Pixmap CurrentPixmap = None;
 char *PixmapPath = NULL;
 
-extern char *__progname;
+static const char *prog_name;
 
 typedef struct BackgroundTexture {
 	int refcount;
@@ -96,11 +97,17 @@ typedef struct BackgroundTexture {
 	int height;
 } BackgroundTexture;
 
+static noreturn void quit(int rcode)
+{
+	WMReleaseApplication();
+	exit(rcode);
+}
+
 static void initXinerama(void)
 {
 	xineInfo.screens = NULL;
 	xineInfo.count = 0;
-#ifdef XINERAMA
+#ifdef USE_XINERAMA
 # ifdef SOLARIS_XINERAMA
 	if (XineramaGetState(dpy, scr)) {
 		XRectangle head[MAXFRAMEBUFFERS];
@@ -137,7 +144,7 @@ static void initXinerama(void)
 		XFree(xine_screens);
 	}
 # endif				/* !SOLARIS_XINERAMA */
-#endif				/* XINERAMA */
+#endif				/* USE_XINERAMA */
 }
 
 static RImage *loadImage(RContext * rc, const char *file)
@@ -220,6 +227,8 @@ applyImage(RContext * rc, BackgroundTexture * texture, RImage * image, char type
 
 			if (!RConvertImage(rc, image, &pixmap)) {
 				wwarning("could not convert texture:%s", RMessageForError(RErrorCode));
+				if (fimage)
+					RReleaseImage(image);
 				return;
 			}
 
@@ -316,7 +325,7 @@ static BackgroundTexture *parseTexture(RContext * rc, char *text)
 		RColor color1, color2;
 		RImage *image;
 		Pixmap pixmap;
-		int gtype;
+		RGradientStyle gtype;
 		int iwidth, iheight;
 
 		GETSTRORGOTO(val, tmp, 1, error);
@@ -386,7 +395,7 @@ static BackgroundTexture *parseTexture(RContext * rc, char *text)
 		RImage *image;
 		Pixmap pixmap;
 		int i, j;
-		int gtype;
+		RGradientStyle gtype;
 		int iwidth, iheight;
 
 		colors = malloc(sizeof(RColor *) * (count - 1));
@@ -514,9 +523,8 @@ static BackgroundTexture *parseTexture(RContext * rc, char *text)
 			rcolor.blue = 0;
 		}
 		/* for images with a transparent color */
-		if (image->data[3]) {
+		if (image && image->data[3])
 			RCombineImageWithColor(image, &rcolor);
-		}
 
 		switch (toupper(type[0])) {
 		case 'T':
@@ -527,8 +535,6 @@ static BackgroundTexture *parseTexture(RContext * rc, char *text)
 				RReleaseImage(image);
 				goto error;
 			}
-			if (image)
-				RReleaseImage(image);
 
 			texture->pixmap = pixmap;
 			texture->color = color;
@@ -548,7 +554,10 @@ static BackgroundTexture *parseTexture(RContext * rc, char *text)
 				texture->width = scrWidth;
 				texture->height = scrHeight;
 
-#ifdef XINERAMA
+				if (!image)
+					break;
+
+#ifdef USE_XINERAMA
 				if (xineInfo.count && ! xineStretch) {
 					int i;
 					for (i = 0; i < xineInfo.count; ++i) {
@@ -560,13 +569,15 @@ static BackgroundTexture *parseTexture(RContext * rc, char *text)
 				} else {
 					applyImage(rc, texture, image, type[0], 0, 0, scrWidth, scrHeight);
 				}
-#else				/* !XINERAMA */
+#else				/* !USE_XINERAMA */
 				applyImage(rc, texture, image, type[0], 0, 0, scrWidth, scrHeight);
-#endif				/* !XINERAMA */
-				RReleaseImage(image);
+#endif				/* !USE_XINERAMA */
 			}
 			break;
 		}
+		if (image)
+			RReleaseImage(image);
+
 	} else if (strcasecmp(type, "thgradient") == 0
 		   || strcasecmp(type, "tvgradient") == 0 || strcasecmp(type, "tdgradient") == 0) {
 		XColor color;
@@ -577,7 +588,7 @@ static BackgroundTexture *parseTexture(RContext * rc, char *text)
 		Pixmap pixmap;
 		int opaq;
 		char *file;
-		int gtype;
+		RGradientStyle gtype;
 		int twidth, theight;
 
 		GETSTRORGOTO(val, file, 1, error);
@@ -636,7 +647,6 @@ static BackgroundTexture *parseTexture(RContext * rc, char *text)
 
 		if (!gradient) {
 			wwarning("could not render texture:%s", RMessageForError(RErrorCode));
-			RReleaseImage(gradient);
 			RReleaseImage(image);
 			goto error;
 		}
@@ -786,6 +796,10 @@ static Pixmap duplicatePixmap(Pixmap pixmap, int width, int height)
 
 static int dummyErrorHandler(Display * dpy, XErrorEvent * err)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) dpy;
+	(void) err;
+
 	return 0;
 }
 
@@ -854,7 +868,6 @@ static int readmsg(int fd, char *buffer, int size)
 {
 	int count;
 
-	count = 0;
 	while (size > 0) {
 		count = read(fd, buffer, size);
 		if (count < 0)
@@ -876,7 +889,7 @@ static int readmsg(int fd, char *buffer, int size)
  * n is 4 bytes
  * size = 4 bytes for length of the message data
  */
-static void helperLoop(RContext * rc)
+static noreturn void helperLoop(RContext * rc)
 {
 	BackgroundTexture *textures[WORKSPACE_COUNT];
 	int maxTextures = 0;
@@ -895,13 +908,21 @@ static void helperLoop(RContext * rc)
 			errcount--;
 			if (errcount == 0) {
 				wfatal("quitting");
-				exit(1);
+				quit(1);
 			}
 			continue;
 		}
 		memcpy(buf, buffer, 4);
 		buf[4] = 0;
 		size = atoi(buf);
+		if (size < 0 || size > sizeof(buffer)) {
+			wfatal("received invalid size %d for message from WindowMaker", size);
+			quit(1);
+		}
+		if (size == 0) {
+			werror("received 0-sized message from WindowMaker, trying to continue");
+			continue;
+		}
 
 		/* get message */
 		if (readmsg(0, buffer, size) < 0) {
@@ -909,7 +930,7 @@ static void helperLoop(RContext * rc)
 			errcount--;
 			if (errcount == 0) {
 				wfatal("quitting");
-				exit(1);
+				quit(1);
 			}
 			continue;
 		}
@@ -965,7 +986,7 @@ static void helperLoop(RContext * rc)
 #ifdef DEBUG
 			printf("exit command\n");
 #endif
-			exit(0);
+			quit(0);
 
 		default:
 			wwarning("unknown message received");
@@ -976,11 +997,16 @@ static void helperLoop(RContext * rc)
 
 static void updateDomain(const char *domain, const char *key, const char *texture)
 {
+	int result;
 	char *program = "wdwrite";
+	char cmd_smooth[1024];
 
-	/* here is a mem leak */
-	system(wstrconcat("wdwrite ",
-			  wstrconcat(domain, smooth ? " SmoothWorkspaceBack YES" : " SmoothWorkspaceBack NO")));
+	snprintf(cmd_smooth, sizeof(cmd_smooth),
+	         "wdwrite %s SmoothWorkspaceBack %s",
+	         domain, smooth ? "YES" : "NO");
+	result = system(cmd_smooth);
+	if (result == -1)
+		werror("error executing system(\"%s\")", cmd_smooth);
 
 	execlp(program, program, domain, key, texture, NULL);
 	wwarning("warning could not run \"%s\"", program);
@@ -1121,38 +1147,32 @@ static char *getFullPixmapPath(const char *file)
 	return wstrdup(file);
 }
 
-void wAbort(void)
-{
-	wfatal("aborting");
-	exit(1);
-}
-
 static void print_help(void)
 {
-	printf("Usage: %s [options] [image]\n", __progname);
+	printf("Usage: %s [options] [image]\n", prog_name);
 	puts("Sets the workspace background to the specified image or a texture and");
 	puts("optionally update Window Maker configuration");
 	puts("");
-	puts(" -display                         display to use");
-	puts(" -d, --dither                     dither image");
-	puts(" -m, --match                      match  colors");
-	puts(" -S, --smooth                     smooth scaled image");
-#ifdef XINERAMA
-	puts(" -X, --xinerama                   stretch image across Xinerama heads");
+	puts(" -display <display>            display to use");
+	puts(" -d, --dither                  dither image");
+	puts(" -m, --match                   match  colors");
+	puts(" -S, --smooth                  smooth scaled image");
+#ifdef USE_XINERAMA
+	puts(" -X, --xinerama                stretch image across Xinerama heads");
 #endif
-	puts(" -b, --back-color <color>         background color");
-	puts(" -t, --tile                       tile   image");
-	puts(" -e, --center                     center image");
-	puts(" -s, --scale                      scale  image (default)");
-	puts(" -a, --maxscale                   scale  image and keep aspect ratio");
-	puts(" -f, --fillscale                  scale  image to fill screen and keep aspect ratio");
-	puts(" -u, --update-wmaker              update WindowMaker domain database");
-	puts(" -D, --update-domain <domain>     update <domain> database");
-	puts(" -c, --colors <cpc>               colors per channel to use");
-	puts(" -p, --parse <texture>            proplist style texture specification");
-	puts(" -w, --workspace <workspace>      update background for the specified workspace");
-	puts(" -v, --version                    show version of wmsetbg and exit");
-	puts(" -h, --help                       show this help and exit");
+	puts(" -b, --back-color <color>      background color");
+	puts(" -t, --tile                    tile   image");
+	puts(" -e, --center                  center image");
+	puts(" -s, --scale                   scale  image (default)");
+	puts(" -a, --maxscale                scale  image and keep aspect ratio");
+	puts(" -f, --fillscale               scale  image to fill screen and keep aspect ratio");
+	puts(" -u, --update-wmaker           update WindowMaker domain database");
+	puts(" -D, --update-domain <domain>  update <domain> database");
+	puts(" -c, --colors <cpc>            colors per channel to use");
+	puts(" -p, --parse <texture>         proplist style texture specification");
+	puts(" -w, --workspace <workspace>   update background for the specified workspace");
+	puts(" -v, --version                 show version of wmsetbg and exit");
+	puts(" -h, --help                    show this help and exit");
 }
 
 static void changeTextureForWorkspace(const char *domain, char *texture, int workspace)
@@ -1183,6 +1203,8 @@ static void changeTextureForWorkspace(const char *domain, char *texture, int wor
 			WMAddToPLArray(array, empty);
 		}
 		WMAddToPLArray(array, val);
+
+		WMReleasePropList(empty);
 	} else {
 		WMDeleteFromPLArray(array, workspace);
 		WMInsertInPLArray(array, workspace, val);
@@ -1190,6 +1212,8 @@ static void changeTextureForWorkspace(const char *domain, char *texture, int wor
 
 	value = WMGetPropListDescription(array, False);
 	updateDomain(domain, "WorkspaceSpecificBack", value);
+
+	WMReleasePropList(array);
 }
 
 int main(int argc, char **argv)
@@ -1202,7 +1226,8 @@ int main(int argc, char **argv)
 	char *back_color = "gray20";
 	char *image_name = NULL;
 	char *domain = "WindowMaker";
-	int update = 0, cpc = 4, render_mode = RDitheredRendering, obey_user = 0;
+	int update = 0, cpc = 4, obey_user = 0;
+	RRenderingMode render_mode = RDitheredRendering;
 	char *texture = NULL;
 	int workspace = -1;
 
@@ -1219,6 +1244,7 @@ int main(int argc, char **argv)
 
 	WMInitializeApplication("wmsetbg", &argc, argv);
 
+	prog_name = argv[0];
 	for (i = 1; i < argc; i++) {
 		if (strcmp(argv[i], "-helper") == 0) {
 			helperMode = 1;
@@ -1226,7 +1252,7 @@ int main(int argc, char **argv)
 			i++;
 			if (i >= argc) {
 				wfatal("too few arguments for %s", argv[i - 1]);
-				exit(1);
+				quit(1);
 			}
 			display = argv[i];
 		} else if (strcmp(argv[i], "-s") == 0 || strcmp(argv[i], "--scale") == 0) {
@@ -1247,7 +1273,7 @@ int main(int argc, char **argv)
 			obey_user++;
 		} else if (strcmp(argv[i], "-S") == 0 || strcmp(argv[i], "--smooth") == 0) {
 			smooth = True;
-#ifdef XINERAMA
+#ifdef USE_XINERAMA
 		} else if (strcmp(argv[i], "-X") == 0 || strcmp(argv[i], "--xinerama") == 0) {
 			xineStretch = True;
 #endif
@@ -1258,61 +1284,61 @@ int main(int argc, char **argv)
 			i++;
 			if (i >= argc) {
 				wfatal("too few arguments for %s", argv[i - 1]);
-				exit(1);
+				quit(1);
 			}
 			domain = wstrdup(argv[i]);
 		} else if (strcmp(argv[i], "-c") == 0 || strcmp(argv[i], "--colors") == 0) {
 			i++;
 			if (i >= argc) {
 				wfatal("too few arguments for %s", argv[i - 1]);
-				exit(1);
+				quit(1);
 			}
 			if (sscanf(argv[i], "%i", &cpc) != 1) {
 				wfatal("bad value for colors per channel: \"%s\"", argv[i]);
-				exit(1);
+				quit(1);
 			}
 		} else if (strcmp(argv[i], "-b") == 0 || strcmp(argv[i], "--back-color") == 0) {
 			i++;
 			if (i >= argc) {
 				wfatal("too few arguments for %s", argv[i - 1]);
-				exit(1);
+				quit(1);
 			}
 			back_color = argv[i];
 		} else if (strcmp(argv[i], "-p") == 0 || strcmp(argv[i], "--parse") == 0) {
 			i++;
 			if (i >= argc) {
 				wfatal("too few arguments for %s", argv[i - 1]);
-				exit(1);
+				quit(1);
 			}
 			texture = argv[i];
 		} else if (strcmp(argv[i], "-w") == 0 || strcmp(argv[i], "--workspace") == 0) {
 			i++;
 			if (i >= argc) {
 				wfatal("too few arguments for %s", argv[i - 1]);
-				exit(1);
+				quit(1);
 			}
 			if (sscanf(argv[i], "%i", &workspace) != 1) {
 				wfatal("bad value for workspace number: \"%s\"", argv[i]);
-				exit(1);
+				quit(1);
 			}
 		} else if (strcmp(argv[i], "-v") == 0 || strcmp(argv[i], "--version") == 0) {
-			printf("%s (Window Maker %s)\n", __progname, VERSION);
-			exit(0);
+			printf("%s (Window Maker %s)\n", prog_name, VERSION);
+			quit(0);
 		} else if (strcmp(argv[i], "-h") == 0 || strcmp(argv[i], "--help") == 0) {
 			print_help();
-			exit(0);
+			quit(0);
 		} else if (argv[i][0] != '-') {
 			image_name = argv[i];
 		} else {
-			printf("%s: invalid argument '%s'\n", __progname, argv[i]);
-			printf("Try '%s --help' for more information\n", __progname);
-			exit(1);
+			printf("%s: invalid argument '%s'\n", prog_name, argv[i]);
+			printf("Try '%s --help' for more information\n", prog_name);
+			quit(1);
 		}
 	}
 	if (!image_name && !texture && !helperMode) {
-		printf("%s: you must specify a image file name or a texture\n", __progname);
-		printf("Try '%s --help' for more information\n", __progname);
-		exit(1);
+		printf("%s: you must specify a image file name or a texture\n", prog_name);
+		printf("Try '%s --help' for more information\n", prog_name);
+		quit(1);
 	}
 
 	PixmapPath = getPixmapPath(domain);
@@ -1332,7 +1358,7 @@ int main(int argc, char **argv)
 	dpy = XOpenDisplay(display);
 	if (!dpy) {
 		wfatal("could not open display");
-		exit(1);
+		quit(1);
 	}
 #if 0
 	XSynchronize(dpy, 1);
@@ -1365,12 +1391,16 @@ int main(int argc, char **argv)
 
 	if (!rc) {
 		wfatal("could not initialize wrlib: %s", RMessageForError(RErrorCode));
-		exit(1);
+		quit(1);
 	}
 
 	if (helperMode) {
+		int result;
+
 		/* lower priority, so that it wont use all the CPU */
-		nice(15);
+		result = nice(15);
+		if (result == -1)
+			wwarning("error could not nice process");
 
 		helperLoop(rc);
 	} else {
@@ -1379,8 +1409,7 @@ int main(int argc, char **argv)
 
 		if (!texture) {
 			char *image_path = getFullPixmapPath(image_name);
-
-			sprintf(buffer, "(%s, \"%s\", %s)", style, image_path, back_color);
+			snprintf(buffer, sizeof(buffer), "(%s, \"%s\", %s)", style, image_path, back_color);
 			wfree(image_path);
 			texture = (char *)buffer;
 		}
@@ -1391,7 +1420,7 @@ int main(int argc, char **argv)
 
 		tex = parseTexture(rc, texture);
 		if (!tex)
-			exit(1);
+			quit(1);
 
 		if (workspace < 0)
 			changeTexture(tex);
@@ -1401,5 +1430,6 @@ int main(int argc, char **argv)
 		}
 	}
 
+	WMReleaseApplication();
 	return 0;
 }
