@@ -35,12 +35,16 @@
 #include <math.h>
 
 #include "wraster.h"
+#include "scale.h"
 
-extern void _wraster_change_filter(int type);
+
+#ifndef HAVE_FLOAT_MATHFUNC
+#define powf(x, y) ((float) pow((double)(x), (double)(y)))
+#endif
 
 static Bool bestContext(Display * dpy, int screen_number, RContext * context);
 
-static RContextAttributes DEFAULT_CONTEXT_ATTRIBS = {
+static const RContextAttributes DEFAULT_CONTEXT_ATTRIBS = {
 	RC_UseSharedMemory | RC_RenderMode | RC_ColorsPerChannel,	/* flags */
 	RDitheredRendering,	/* render_mode */
 	4,			/* colors_per_channel */
@@ -142,187 +146,13 @@ static Bool setupStandardColormap(RContext * ctx, Atom property)
 	return True;
 }
 
-static Bool allocatePseudoColor(RContext * ctx)
+static XColor *allocateColor(RContext *ctx, XColor *colors, int ncolors)
 {
-	XColor *colors;
 	XColor avcolors[256];
 	int avncolors;
-	int i, ncolors, r, g, b;
+	int i, r, g, b;
 	int retries;
-	int cpc = ctx->attribs->colors_per_channel;
 
-	ncolors = cpc * cpc * cpc;
-
-	if (ncolors > (1 << ctx->depth)) {
-		/* reduce colormap size */
-		cpc = ctx->attribs->colors_per_channel = 1 << ((int)ctx->depth / 3);
-		ncolors = cpc * cpc * cpc;
-	}
-
-	assert(cpc >= 2 && ncolors <= (1 << ctx->depth));
-
-	colors = malloc(sizeof(XColor) * ncolors);
-	if (!colors) {
-		RErrorCode = RERR_NOMEMORY;
-		return False;
-	}
-
-	ctx->pixels = malloc(sizeof(unsigned long) * ncolors);
-	if (!ctx->pixels) {
-		free(colors);
-		RErrorCode = RERR_NOMEMORY;
-		return False;
-	}
-
-	i = 0;
-
-	if ((ctx->attribs->flags & RC_GammaCorrection) && ctx->attribs->rgamma > 0
-	    && ctx->attribs->ggamma > 0 && ctx->attribs->bgamma > 0) {
-		double rg, gg, bg;
-		double tmp;
-
-		/* do gamma correction */
-		rg = 1.0 / ctx->attribs->rgamma;
-		gg = 1.0 / ctx->attribs->ggamma;
-		bg = 1.0 / ctx->attribs->bgamma;
-		for (r = 0; r < cpc; r++) {
-			for (g = 0; g < cpc; g++) {
-				for (b = 0; b < cpc; b++) {
-					colors[i].red = (r * 0xffff) / (cpc - 1);
-					colors[i].green = (g * 0xffff) / (cpc - 1);
-					colors[i].blue = (b * 0xffff) / (cpc - 1);
-					colors[i].flags = DoRed | DoGreen | DoBlue;
-
-					tmp = (double)colors[i].red / 65536.0;
-					colors[i].red = (unsigned short)(65536.0 * pow(tmp, rg));
-
-					tmp = (double)colors[i].green / 65536.0;
-					colors[i].green = (unsigned short)(65536.0 * pow(tmp, gg));
-
-					tmp = (double)colors[i].blue / 65536.0;
-					colors[i].blue = (unsigned short)(65536.0 * pow(tmp, bg));
-
-					i++;
-				}
-			}
-		}
-
-	} else {
-		for (r = 0; r < cpc; r++) {
-			for (g = 0; g < cpc; g++) {
-				for (b = 0; b < cpc; b++) {
-					colors[i].red = (r * 0xffff) / (cpc - 1);
-					colors[i].green = (g * 0xffff) / (cpc - 1);
-					colors[i].blue = (b * 0xffff) / (cpc - 1);
-					colors[i].flags = DoRed | DoGreen | DoBlue;
-					i++;
-				}
-			}
-		}
-	}
-	/* try to allocate the colors */
-	for (i = 0; i < ncolors; i++) {
-		if (!XAllocColor(ctx->dpy, ctx->cmap, &(colors[i]))) {
-			colors[i].flags = 0;	/* failed */
-		} else {
-			colors[i].flags = DoRed | DoGreen | DoBlue;
-		}
-	}
-	/* try to allocate close values for the colors that couldn't
-	 * be allocated before */
-	avncolors = (1 << ctx->depth > 256 ? 256 : 1 << ctx->depth);
-	for (i = 0; i < avncolors; i++)
-		avcolors[i].pixel = i;
-
-	XQueryColors(ctx->dpy, ctx->cmap, avcolors, avncolors);
-
-	for (i = 0; i < ncolors; i++) {
-		if (colors[i].flags == 0) {
-			int j;
-			unsigned long cdiff = 0xffffffff, diff;
-			unsigned long closest = 0;
-
-			retries = 2;
-
-			while (retries--) {
-				/* find closest color */
-				for (j = 0; j < avncolors; j++) {
-					r = (colors[i].red - avcolors[i].red) >> 8;
-					g = (colors[i].green - avcolors[i].green) >> 8;
-					b = (colors[i].blue - avcolors[i].blue) >> 8;
-					diff = r * r + g * g + b * b;
-					if (diff < cdiff) {
-						cdiff = diff;
-						closest = j;
-					}
-				}
-				/* allocate closest color found */
-				colors[i].red = avcolors[closest].red;
-				colors[i].green = avcolors[closest].green;
-				colors[i].blue = avcolors[closest].blue;
-				if (XAllocColor(ctx->dpy, ctx->cmap, &colors[i])) {
-					colors[i].flags = DoRed | DoGreen | DoBlue;
-					break;	/* succeeded, don't need to retry */
-				}
-#ifdef WRLIB_DEBUG
-				fputs("close color allocation failed. Retrying...\n", stderr);
-#endif
-			}
-		}
-	}
-
-	ctx->colors = colors;
-	ctx->ncolors = ncolors;
-
-	/* fill the pixels shortcut array */
-	for (i = 0; i < ncolors; i++) {
-		ctx->pixels[i] = ctx->colors[i].pixel;
-	}
-
-	return True;
-}
-
-static XColor *allocateGrayScale(RContext * ctx)
-{
-	XColor *colors;
-	XColor avcolors[256];
-	int avncolors;
-	int i, ncolors, r, g, b;
-	int retries;
-	int cpc = ctx->attribs->colors_per_channel;
-
-	ncolors = cpc * cpc * cpc;
-
-	if (ctx->vclass == StaticGray) {
-		/* we might as well use all grays */
-		ncolors = 1 << ctx->depth;
-	} else {
-		if (ncolors > (1 << ctx->depth)) {
-			/* reduce colormap size */
-			cpc = ctx->attribs->colors_per_channel = 1 << ((int)ctx->depth / 3);
-			ncolors = cpc * cpc * cpc;
-		}
-
-		assert(cpc >= 2 && ncolors <= (1 << ctx->depth));
-	}
-
-	if (ncolors >= 256 && ctx->vclass == StaticGray) {
-		/* don't need dithering for 256 levels of gray in StaticGray visual */
-		ctx->attribs->render_mode = RBestMatchRendering;
-	}
-
-	colors = malloc(sizeof(XColor) * ncolors);
-	if (!colors) {
-		RErrorCode = RERR_NOMEMORY;
-		return False;
-	}
-	for (i = 0; i < ncolors; i++) {
-		colors[i].red = (i * 0xffff) / (ncolors - 1);
-		colors[i].green = (i * 0xffff) / (ncolors - 1);
-		colors[i].blue = (i * 0xffff) / (ncolors - 1);
-		colors[i].flags = DoRed | DoGreen | DoBlue;
-	}
-	/* try to allocate the colors */
 	for (i = 0; i < ncolors; i++) {
 #ifdef WRLIB_DEBUG
 		fprintf(stderr, "trying:%x,%x,%x\n", colors[i].red, colors[i].green, colors[i].blue);
@@ -389,6 +219,135 @@ static XColor *allocateGrayScale(RContext * ctx)
 	return colors;
 }
 
+static Bool allocatePseudoColor(RContext *ctx)
+{
+	XColor *colors;
+	int i, ncolors, r, g, b;
+	int cpc = ctx->attribs->colors_per_channel;
+
+	ncolors = cpc * cpc * cpc;
+
+	if (ncolors > (1 << ctx->depth)) {
+		/* reduce colormap size */
+		cpc = ctx->attribs->colors_per_channel = 1 << ((int)ctx->depth / 3);
+		ncolors = cpc * cpc * cpc;
+	}
+
+	assert(cpc >= 2 && ncolors <= (1 << ctx->depth));
+
+	colors = malloc(sizeof(XColor) * ncolors);
+	if (!colors) {
+		RErrorCode = RERR_NOMEMORY;
+		return False;
+	}
+
+	ctx->pixels = malloc(sizeof(unsigned long) * ncolors);
+	if (!ctx->pixels) {
+		free(colors);
+		RErrorCode = RERR_NOMEMORY;
+		return False;
+	}
+
+	i = 0;
+
+	if ((ctx->attribs->flags & RC_GammaCorrection) && ctx->attribs->rgamma > 0
+	    && ctx->attribs->ggamma > 0 && ctx->attribs->bgamma > 0) {
+		float rg, gg, bg;
+		float tmp;
+
+		/* do gamma correction */
+		rg = 1.0F / ctx->attribs->rgamma;
+		gg = 1.0F / ctx->attribs->ggamma;
+		bg = 1.0F / ctx->attribs->bgamma;
+		for (r = 0; r < cpc; r++) {
+			for (g = 0; g < cpc; g++) {
+				for (b = 0; b < cpc; b++) {
+					colors[i].red = (r * 0xffff) / (cpc - 1);
+					colors[i].green = (g * 0xffff) / (cpc - 1);
+					colors[i].blue = (b * 0xffff) / (cpc - 1);
+					colors[i].flags = DoRed | DoGreen | DoBlue;
+
+					tmp = (float) colors[i].red / 65536.0F;
+					colors[i].red = (unsigned short)(65536.0F * powf(tmp, rg));
+
+					tmp = (float) colors[i].green / 65536.0F;
+					colors[i].green = (unsigned short)(65536.0F * powf(tmp, gg));
+
+					tmp = (float) colors[i].blue / 65536.0F;
+					colors[i].blue = (unsigned short)(65536.0F * powf(tmp, bg));
+
+					i++;
+				}
+			}
+		}
+
+	} else {
+		for (r = 0; r < cpc; r++) {
+			for (g = 0; g < cpc; g++) {
+				for (b = 0; b < cpc; b++) {
+					colors[i].red = (r * 0xffff) / (cpc - 1);
+					colors[i].green = (g * 0xffff) / (cpc - 1);
+					colors[i].blue = (b * 0xffff) / (cpc - 1);
+					colors[i].flags = DoRed | DoGreen | DoBlue;
+					i++;
+				}
+			}
+		}
+	}
+	/* try to allocate the colors */
+	ctx->colors = allocateColor(ctx, colors, ncolors);
+	ctx->ncolors = ncolors;
+
+	/* fill the pixels shortcut array */
+	for (i = 0; i < ncolors; i++) {
+		ctx->pixels[i] = ctx->colors[i].pixel;
+	}
+
+	return True;
+}
+
+static XColor *allocateGrayScale(RContext * ctx)
+{
+	XColor *colors;
+	int i, ncolors;
+	int cpc = ctx->attribs->colors_per_channel;
+
+	ncolors = cpc * cpc * cpc;
+
+	if (ctx->vclass == StaticGray) {
+		/* we might as well use all grays */
+		ncolors = 1 << ctx->depth;
+	} else {
+		if (ncolors > (1 << ctx->depth)) {
+			/* reduce colormap size */
+			cpc = ctx->attribs->colors_per_channel = 1 << ((int)ctx->depth / 3);
+			ncolors = cpc * cpc * cpc;
+		}
+
+		assert(cpc >= 2 && ncolors <= (1 << ctx->depth));
+	}
+
+	if (ncolors >= 256 && ctx->vclass == StaticGray) {
+		/* don't need dithering for 256 levels of gray in StaticGray visual */
+		ctx->attribs->render_mode = RBestMatchRendering;
+	}
+
+	colors = malloc(sizeof(XColor) * ncolors);
+	if (!colors) {
+		RErrorCode = RERR_NOMEMORY;
+		return False;
+	}
+	for (i = 0; i < ncolors; i++) {
+		colors[i].red = (i * 0xffff) / (ncolors - 1);
+		colors[i].green = (i * 0xffff) / (ncolors - 1);
+		colors[i].blue = (i * 0xffff) / (ncolors - 1);
+		colors[i].flags = DoRed | DoGreen | DoBlue;
+	}
+
+	/* try to allocate the colors */
+	return allocateColor(ctx, colors, ncolors);
+}
+
 static Bool setupPseudoColorColormap(RContext * context)
 {
 	Atom property = 0;
@@ -483,7 +442,7 @@ static void gatherconfig(RContext * context, int screen_n)
 	ptr = mygetenv("WRASTER_GAMMA", screen_n);
 	if (ptr) {
 		float g1, g2, g3;
-		if (sscanf(ptr, "%f/%f/%f", &g1, &g2, &g3) != 3 || g1 <= 0.0 || g2 <= 0.0 || g3 <= 0.0) {
+		if (sscanf(ptr, "%f/%f/%f", &g1, &g2, &g3) != 3 || g1 <= 0.0F || g2 <= 0.0F || g3 <= 0.0F) {
 			printf("wrlib: invalid value(s) for gamma correction \"%s\"\n", ptr);
 		} else {
 			context->attribs->flags |= RC_GammaCorrection;
@@ -502,14 +461,6 @@ static void gatherconfig(RContext * context, int screen_n)
 			context->attribs->colors_per_channel = i;
 		}
 	}
-
-	ptr = mygetenv("WRASTER_OPTIMIZE_FOR_SPEED", screen_n);
-	if (ptr) {
-		context->flags.optimize_for_speed = 1;
-	} else {
-		context->flags.optimize_for_speed = 0;
-	}
-
 }
 
 static void getColormap(RContext * context, int screen_number)
@@ -596,7 +547,7 @@ RContext *RCreateContext(Display * dpy, int screen_number, const RContextAttribu
 
 	/* get configuration from environment variables */
 	gatherconfig(context, screen_number);
-	_wraster_change_filter(context->attribs->scaling_filter);
+	wraster_change_filter(context->attribs->scaling_filter);
 	if ((context->attribs->flags & RC_VisualID)) {
 		XVisualInfo *vinfo, templ;
 		int nret;
@@ -672,7 +623,7 @@ RContext *RCreateContext(Display * dpy, int screen_number, const RContextAttribu
 	}
 
 	/* check avaiability of MIT-SHM */
-#ifdef XSHM
+#ifdef USE_XSHM
 	if (!(context->attribs->flags & RC_UseSharedMemory)) {
 		context->attribs->flags |= RC_UseSharedMemory;
 		context->attribs->use_shared_memory = True;
@@ -696,6 +647,21 @@ RContext *RCreateContext(Display * dpy, int screen_number, const RContextAttribu
 	return context;
 }
 
+void RDestroyContext(RContext *context)
+{
+	if (context) {
+		if (context->copy_gc)
+			XFreeGC(context->dpy, context->copy_gc);
+		if (context->attribs) {
+			if ((context->attribs->flags & RC_VisualID) &&
+			    !(context->attribs->flags & RC_DefaultVisual))
+				XDestroyWindow(context->dpy, context->drawable);
+			free(context->attribs);
+		}
+		free(context);
+	}
+}
+
 static Bool bestContext(Display * dpy, int screen_number, RContext * context)
 {
 	XVisualInfo *vinfo = NULL, rvinfo;
