@@ -34,10 +34,9 @@
 #include <assert.h>
 
 #include "wraster.h"
+#include "convert.h"
+#include "xutil.h"
 
-#ifdef XSHM
-extern Pixmap R_CreateXImageMappedPixmap(RContext * context, RXImage * ximage);
-#endif
 
 #define NFREE(n)  if (n) free(n)
 
@@ -62,6 +61,38 @@ typedef struct RStdConversionTable {
 static RConversionTable *conversionTable = NULL;
 static RStdConversionTable *stdConversionTable = NULL;
 
+static void release_conversion_table(void)
+{
+	RConversionTable *tmp = conversionTable;
+
+	while (tmp) {
+		RConversionTable *tmp_to_delete = tmp;
+
+		tmp = tmp->next;
+		free(tmp_to_delete);
+	}
+	conversionTable = NULL;
+}
+
+static void release_std_conversion_table(void)
+{
+	RStdConversionTable *tmp = stdConversionTable;
+
+	while (tmp) {
+		RStdConversionTable *tmp_to_delete = tmp;
+
+		tmp = tmp->next;
+		free(tmp_to_delete);
+	}
+	stdConversionTable = NULL;
+}
+
+void r_destroy_conversion_tables(void)
+{
+	release_conversion_table();
+	release_std_conversion_table();
+}
+
 static unsigned short *computeTable(unsigned short mask)
 {
 	RConversionTable *tmp = conversionTable;
@@ -292,7 +323,8 @@ static RXImage *image2TrueColor(RContext * ctx, RImage * image)
 	}
 
 	if (ctx->attribs->render_mode == RBestMatchRendering) {
-		int ofs, r, g, b;
+		int ofs;
+		unsigned long r, g, b;
 		int x, y;
 		unsigned long pixel;
 		unsigned char *ptr = image->data;
@@ -305,7 +337,10 @@ static RXImage *image2TrueColor(RContext * ctx, RImage * image)
 			for (y = 0; y < image->height; y++) {
 				for (x = 0; x < image->width; x++, ptr += channels) {
 					/* reduce pixel */
-					pixel = (*(ptr) << roffs) | (*(ptr + 1) << goffs) | (*(ptr + 2) << boffs);
+					r = ptr[0];
+					g = ptr[1];
+					b = ptr[2];
+					pixel = (r << roffs) | (g << goffs) | (b << boffs);
 					XPutPixel(ximg->image, x, y, pixel);
 				}
 			}
@@ -819,7 +854,7 @@ static RXImage *image2Bitmap(RContext * ctx, RImage * image, int threshold)
 int RConvertImage(RContext * context, RImage * image, Pixmap * pixmap)
 {
 	RXImage *ximg = NULL;
-#ifdef XSHM
+#ifdef USE_XSHM
 	Pixmap tmp;
 #endif
 
@@ -852,7 +887,7 @@ int RConvertImage(RContext * context, RImage * image, Pixmap * pixmap)
 
 	*pixmap = XCreatePixmap(context->dpy, context->drawable, image->width, image->height, context->depth);
 
-#ifdef XSHM
+#ifdef USE_XSHM
 	if (context->flags.use_shared_pixmap && ximg->is_shared)
 		tmp = R_CreateXImageMappedPixmap(context, ximg);
 	else
@@ -873,9 +908,9 @@ int RConvertImage(RContext * context, RImage * image, Pixmap * pixmap)
 	} else {
 		RPutXImage(context, *pixmap, context->copy_gc, ximg, 0, 0, 0, 0, image->width, image->height);
 	}
-#else				/* !XSHM */
+#else				/* !USE_XSHM */
 	RPutXImage(context, *pixmap, context->copy_gc, ximg, 0, 0, 0, 0, image->width, image->height);
-#endif				/* !XSHM */
+#endif				/* !USE_XSHM */
 
 	RDestroyXImage(context, ximg);
 
@@ -940,8 +975,9 @@ Bool RGetClosestXColor(RContext * context, const RColor * color, XColor * retCol
 		gtable = computeTable(gmask);
 		btable = computeTable(bmask);
 
-		retColor->pixel = (rtable[color->red] << roffs) |
-		    (gtable[color->green] << goffs) | (btable[color->blue] << boffs);
+		retColor->pixel = (((unsigned long) rtable[color->red])   << roffs)
+		               |  (((unsigned long) gtable[color->green]) << goffs)
+		               |  (((unsigned long) btable[color->blue])  << boffs);
 
 		retColor->red = color->red << 8;
 		retColor->green = color->green << 8;
