@@ -25,24 +25,22 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+
 #include <X11/Xlib.h>
+
 #include "wraster.h"
+#include "rotate.h"
 
 #include <math.h>
 
-#ifndef PI
-#define PI 3.14159265358979323846
-#endif
 
-static RImage *rotateImage(RImage * image, float angle);
+static RImage *rotate_image_90(RImage *source);
+static RImage *rotate_image_270(RImage *source);
+static RImage *rotate_image_any(RImage *source, float angle);
 
-RImage *RRotateImage(RImage * image, float angle)
+
+RImage *RRotateImage(RImage *image, float angle)
 {
-	RImage *img;
-	int nwidth, nheight;
-	int x, y;
-	int bpp = image->format == RRGBAFormat ? 4 : 3;
-
 	/*
 	 * Angle steps below this value would represent a rotation
 	 * of less than 1 pixel for a 4k wide image, so not worth
@@ -50,148 +48,177 @@ RImage *RRotateImage(RImage * image, float angle)
 	 * candidate for an Epsilon when trying to compare angle
 	 * to known values
 	 */
-	static const float min_usable_angle = 0.00699;
+	static const float min_usable_angle = 0.00699F;
 
 	angle = fmod(angle, 360.0);
-	if (angle < 0.0)
-		angle += 360.0;
+	if (angle < 0.0F)
+		angle += 360.0F;
 
 	if (angle < min_usable_angle) {
 		/* Rotate by 0 degree */
 		return RCloneImage(image);
 
-	} else if ((angle > 90.0 - min_usable_angle) &&
-				  (angle < 90.0 + min_usable_angle)) {
-		nwidth = image->height;
-		nheight = image->width;
+	} else if ((angle > 90.0F - min_usable_angle) &&
+				  (angle < 90.0F + min_usable_angle)) {
+		return rotate_image_90(image);
 
-		img = RCreateImage(nwidth, nheight, True);
-		if (!img) {
-			return NULL;
-		}
+	} else if ((angle > 180.0F - min_usable_angle) &&
+				  (angle < 180.0F + min_usable_angle)) {
+		return wraster_rotate_image_180(image);
 
-		if (bpp == 3) {
-			unsigned char *optr, *nptr;
-			unsigned offs;
+	} else if ((angle > 270.0F - min_usable_angle) &&
+				  (angle < 270.0F + min_usable_angle)) {
+		return rotate_image_270(image);
 
-			offs = nwidth * 4;
-
-			optr = image->data;
-			nptr = img->data;
-
-			for (x = 0; x < nwidth; x++) {
-				nptr = img->data + x * 4;
-				for (y = nheight; y; y--) {
-					nptr[0] = *optr++;
-					nptr[1] = *optr++;
-					nptr[2] = *optr++;
-					nptr[3] = 255;
-
-					nptr += offs;
-				}
-			}
-		} else {
-			unsigned *optr, *nptr;
-			unsigned *p;
-
-			optr = (unsigned *)image->data;
-			p = (unsigned *)img->data;
-			for (x = 0; x < nwidth; x++) {
-				nptr = p++;
-				for (y = nheight; y; y--) {
-					*nptr = *optr++;
-					nptr += nwidth;
-				}
-			}
-		}
-	} else if ((angle > 180.0 - min_usable_angle) &&
-				  (angle < 180.0 + min_usable_angle)) {
-
-		nwidth = image->width;
-		nheight = image->height;
-		img = RCreateImage(nwidth, nheight, True);
-		if (!img) {
-			return NULL;
-		}
-
-		if (bpp == 3) {
-			unsigned char *optr, *nptr;
-
-			optr = image->data;
-			nptr = img->data + nwidth * nheight * 4 - 4;
-
-			for (y = 0; y < nheight; y++) {
-				for (x = 0; x < nwidth; x++) {
-					nptr[0] = optr[0];
-					nptr[1] = optr[1];
-					nptr[2] = optr[2];
-					nptr[3] = 255;
-
-					optr += 3;
-					nptr -= 4;
-				}
-			}
-		} else {
-			unsigned *optr, *nptr;
-
-			optr = (unsigned *)image->data;
-			nptr = (unsigned *)img->data + nwidth * nheight - 1;
-
-			for (y = nheight * nwidth - 1; y >= 0; y--) {
-				*nptr = *optr;
-				optr++;
-				nptr--;
-			}
-		}
-	} else if ((angle > 270.0 - min_usable_angle) &&
-				  (angle < 270.0 + min_usable_angle)) {
-		nwidth = image->height;
-		nheight = image->width;
-
-		img = RCreateImage(nwidth, nheight, True);
-		if (!img) {
-			return NULL;
-		}
-
-		if (bpp == 3) {
-			unsigned char *optr, *nptr;
-			unsigned offs;
-
-			offs = nwidth * 4;
-
-			optr = image->data;
-			nptr = img->data;
-
-			for (x = 0; x < nwidth; x++) {
-				nptr = img->data + x * 4;
-				for (y = nheight; y; y--) {
-					nptr[0] = *optr++;
-					nptr[1] = *optr++;
-					nptr[2] = *optr++;
-					nptr[3] = 255;
-
-					nptr += offs;
-				}
-			}
-		} else {
-			unsigned *optr, *nptr;
-			unsigned *p;
-
-			optr = (unsigned *)image->data;
-			p = (unsigned *)img->data + nwidth * nheight;
-			for (x = 0; x < nwidth; x++) {
-				nptr = p--;
-				for (y = nheight; y; y--) {
-					*nptr = *optr++;
-					nptr -= nwidth;
-				}
-			}
-		}
 	} else {
-		img = rotateImage(image, angle);
+		return rotate_image_any(image, angle);
+	}
+}
+
+static RImage *rotate_image_90(RImage *source)
+{
+	RImage *target;
+	int nwidth, nheight;
+	int x, y;
+
+	nwidth = source->height;
+	nheight = source->width;
+
+	target = RCreateImage(nwidth, nheight, (source->format != RRGBFormat));
+	if (!target)
+		return NULL;
+
+	if (source->format == RRGBFormat) {
+		unsigned char *optr, *nptr;
+
+		optr = source->data;
+		for (x = nwidth; x; x--) {
+			nptr = target->data + 3 * (x - 1);
+			for (y = nheight; y; y--) {
+				nptr[0] = *optr++;
+				nptr[1] = *optr++;
+				nptr[2] = *optr++;
+
+				nptr += 3 * nwidth;
+			}
+		}
+
+	} else {
+		unsigned char *optr, *nptr;
+
+		optr = source->data;
+		for (x = nwidth; x; x--) {
+			nptr = target->data + 4 * (x - 1);
+			for (y = nheight; y; y--) {
+				nptr[0] = *optr++;
+				nptr[1] = *optr++;
+				nptr[2] = *optr++;
+				nptr[3] = *optr++;
+
+				nptr += 4 * nwidth;
+			}
+		}
 	}
 
-	return img;
+	return target;
+}
+
+RImage *wraster_rotate_image_180(RImage *source)
+{
+	RImage *target;
+	int nwidth, nheight;
+	int x, y;
+
+	nwidth = source->width;
+	nheight = source->height;
+
+	target = RCreateImage(nwidth, nheight, (source->format != RRGBFormat));
+	if (!target)
+		return NULL;
+
+	if (source->format == RRGBFormat) {
+		unsigned char *optr, *nptr;
+
+		optr = source->data;
+		nptr = target->data + nwidth * nheight * 3 - 3;
+
+		for (y = 0; y < nheight; y++) {
+			for (x = 0; x < nwidth; x++) {
+				nptr[0] = optr[0];
+				nptr[1] = optr[1];
+				nptr[2] = optr[2];
+
+				optr += 3;
+				nptr -= 3;
+			}
+		}
+
+	} else {
+		unsigned char *optr, *nptr;
+
+		optr = source->data;
+		nptr = target->data + nwidth * nheight * 4 - 4;
+
+		for (y = nheight * nwidth - 1; y >= 0; y--) {
+			nptr[0] = optr[0];
+			nptr[1] = optr[1];
+			nptr[2] = optr[2];
+			nptr[3] = optr[3];
+
+			optr += 4;
+			nptr -= 4;
+		}
+	}
+
+	return target;
+}
+
+static RImage *rotate_image_270(RImage *source)
+{
+	RImage *target;
+	int nwidth, nheight;
+	int x, y;
+
+	nwidth = source->height;
+	nheight = source->width;
+
+	target = RCreateImage(nwidth, nheight, (source->format != RRGBFormat));
+	if (!target)
+		return NULL;
+
+	if (source->format == RRGBFormat) {
+		unsigned char *optr, *nptr;
+
+		optr = source->data;
+		for (x = nwidth; x; x--) {
+			nptr = target->data + 3 * nwidth * nheight - x * 3;
+			for (y = nheight; y; y--) {
+				nptr[0] = *optr++;
+				nptr[1] = *optr++;
+				nptr[2] = *optr++;
+
+				nptr -= 3 * nwidth;
+			}
+		}
+
+	} else {
+		unsigned char *optr, *nptr;
+
+		optr = source->data;
+		for (x = nwidth; x; x--) {
+			nptr = target->data + 4 * nwidth * nheight - x * 4;
+			for (y = nheight; y; y--) {
+				nptr[0] = *optr++;
+				nptr[1] = *optr++;
+				nptr[2] = *optr++;
+				nptr[3] = *optr++;
+
+				nptr -= 4 * nwidth;
+			}
+		}
+	}
+
+	return target;
 }
 
 /*
@@ -211,10 +238,11 @@ RImage *RRotateImage(RImage * image, float angle)
  * for each point P1 in the line from C to A
  *	for each point P2 in the perpendicular line starting at P1
  *		get pixel from the source and plot at P2
- * 		increment pixel location from source
+ *		increment pixel location from source
  *
  */
 
+#if 0
 static void
 copyLine(int x1, int y1, int x2, int y2, int nwidth, int format, unsigned char *dst, unsigned char **src)
 {
@@ -294,9 +322,14 @@ copyLine(int x1, int y1, int x2, int y2, int nwidth, int format, unsigned char *
 
 	*src = s;
 }
+#endif
 
-static RImage *rotateImage(RImage * image, float angle)
+static RImage *rotate_image_any(RImage *source, float angle)
 {
+	(void) angle;
+	puts("NOT FULLY IMPLEMENTED");
+	return RCloneImage(source);
+#if 0
 	RImage *img;
 	int nwidth, nheight;
 	int x1, y1;
@@ -308,16 +341,16 @@ static RImage *rotateImage(RImage * image, float angle)
 	int dpr, dpru, p;
 
 	/* only 180o for now */
-	if (angle > 180.0)
-		angle -= 180.0;
+	if (angle > 180.0F)
+		angle -= 180.0F;
 
-	angle = (angle * PI) / 180.0;
+	angle = (angle * WM_PI) / 180.0;
 
 	nwidth = ceil(abs(cos(angle) * image->width))
-	    + ceil(abs(cos(PI / 2 - angle) * image->width));
+	    + ceil(abs(cos(WM_PI / 2 - angle) * image->width));
 
 	nheight = ceil(abs(sin(angle) * image->height))
-	    + ceil(abs(cos(PI / 2 - angle) * image->height));
+	    + ceil(abs(cos(WM_PI / 2 - angle) * image->height));
 
 	img = RCreateImage(nwidth, nheight, True);
 	if (!img)
@@ -326,16 +359,16 @@ static RImage *rotateImage(RImage * image, float angle)
 	src = image->data;
 	dst = img->data;
 
-	x1 = floor(abs(cos(PI / 2 - angle) * image->width));
+	x1 = floor(abs(cos(WM_PI / 2 - angle) * image->width));
 	y1 = 0;
 
 	x2 = 0;
-	y2 = floor(abs(sin(PI / 2 - angle) * image->width));
+	y2 = floor(abs(sin(WM_PI / 2 - angle) * image->width));
 
 	xx = floor(abs(cos(angle) * image->height)) - 1;
 	yy = nheight - 1;
 
-	printf("%ix%i, %i %i     %i %i %i\n", nwidth, nheight, x1, y1, x2, y2, (int)((angle * 180.0) / PI));
+	printf("%ix%i, %i %i     %i %i %i\n", nwidth, nheight, x1, y1, x2, y2, (int)((angle * 180.0) / WM_PI));
 
 	dx = abs(x2 - x1);
 	dy = abs(y2 - y1);
@@ -373,15 +406,13 @@ static RImage *rotateImage(RImage * image, float angle)
 			}
 		}
 	} else {
-		puts("NOT IMPLEMTENED");
-		return img;
 		dpr = dx << 1;
 		dpru = dpr - (dy << 1);
 		p = dpr - dy;
 
 		while (dy-- >= 0) {
-			xx = abs(x1 * sin(angle * PI / 180.0));
-			yy = abs(y1 * cos(angle * PI / 180.0));
+			xx = abs(x1 * sin(angle * WM_PI / 180.0));
+			yy = abs(y1 * cos(angle * WM_PI / 180.0));
 
 			copyLine(x1, y1, xx, yy, nwidth, image->format, dst, &src);
 
@@ -398,4 +429,5 @@ static RImage *rotateImage(RImage * image, float angle)
 	}
 
 	return img;
+#endif
 }
